"""The module implementing the calculating the HMAC message authentication code in
   accordance with R 50.1.113-2016.
"""

__title__ = 'gosthmac'
__author__ = 'Evgeny Drobotun'
__author_email__ = 'drobotun@xakep.ru'
__license__ = 'MIT'
__copyright__ = 'Copyright (C) 2020 Evgeny Drobotun'

from .r_50_1_113_2016 import *



"""The module that implements processes for creating and verifying an electronic digital
   signature according to GOST 34.10-2012.
"""

__title__ = 'gostsignature'
__author__ = 'Evgeny Drobotun'
__author_email__ = 'drobotun@xakep.ru'
__license__ = 'MIT'
__copyright__ = 'Copyright (C) 2020 Evgeny Drobotun'

from .gost_34_10_2012 import *



import unittest
import gostcrypto

TEST_KEY = bytearray.fromhex('000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1f')
TEST_DATA = bytearray.fromhex('0126bdb87800af214341456563780100')
TEST_HMAC_256 = bytearray.fromhex('a1aa5f7de402d7b3d323f2991c8d4534013137010a83754fd0af6d7cd4922ed9')
TEST_HMAC_512 = bytearray.fromhex('a59bab22ecae19c65fbde6e5f4e9f5d8549d31f037f9df9b905500e171923a773d5f1530f2ed7e964cb2eedc29e9ad2f3afe93b2814f79f5000ffc0366c251e6')
TEST_HMAC_256_DOUBLE_UPDATE = bytearray.fromhex('7c3f66aacd8015751cd8c4735819dfa3a2ec36d89c241c6551878c37c84b092e')

class TestHMAC(unittest.TestCase):

    def test_digest_256(self):
        test_hmac = gostcrypto.gosthmac.new('HMAC_GOSTR3411_2012_256', TEST_KEY)
        test_hmac.update(TEST_DATA)
        self.assertEqual(test_hmac.digest(), TEST_HMAC_256)

    def test_digest_512(self):
        test_hmac = gostcrypto.gosthmac.new('HMAC_GOSTR3411_2012_512', TEST_KEY)
        test_hmac.update(TEST_DATA)
        self.assertEqual(test_hmac.digest(), TEST_HMAC_512)

    def test_hexdigest_256(self):
        test_hmac = gostcrypto.gosthmac.new('HMAC_GOSTR3411_2012_256', TEST_KEY)
        test_hmac.update(TEST_DATA)
        self.assertEqual(test_hmac.hexdigest(), TEST_HMAC_256.hex())

    def test_hexdigest_512(self):
        test_hmac = gostcrypto.gosthmac.new('HMAC_GOSTR3411_2012_512', TEST_KEY)
        test_hmac.update(TEST_DATA)
        self.assertEqual(test_hmac.hexdigest(), TEST_HMAC_512.hex())

    def test_copy(self):
        test_hmac = gostcrypto.gosthmac.new('HMAC_GOSTR3411_2012_512', TEST_KEY)
        test_hmac_copy = test_hmac.copy()
        test_hmac_copy.update(TEST_DATA)
        self.assertEqual(test_hmac_copy.hexdigest(), TEST_HMAC_512.hex())

    def test_name(self):
        test_hmac = gostcrypto.gosthmac.new('HMAC_GOSTR3411_2012_512', TEST_KEY)
        self.assertEqual(test_hmac.name, 'HMAC_GOSTR3411_2012_512')
        test_hmac = gostcrypto.gosthmac.new('HMAC_GOSTR3411_2012_256', TEST_KEY)
        self.assertEqual(test_hmac.name, 'HMAC_GOSTR3411_2012_256')

    def test_digest_size(self):
        test_hmac = gostcrypto.gosthmac.new('HMAC_GOSTR3411_2012_512', TEST_KEY)
        self.assertEqual(test_hmac.digest_size, 64)
        test_hmac = gostcrypto.gosthmac.new('HMAC_GOSTR3411_2012_256', TEST_KEY)
        self.assertEqual(test_hmac.digest_size, 32)

    def test_block_size(self):
        test_hmac = gostcrypto.gosthmac.new('HMAC_GOSTR3411_2012_512', TEST_KEY)
        self.assertEqual(test_hmac.block_size, 64)

    def test_key_size_raises(self):
        add = bytearray.fromhex('000102030405060708090a0b0c0d0e0f101112131415161718191a1b1c1d1e1fff')
        with self.assertRaises(ValueError) as context:
            test_hmac = gostcrypto.gosthmac.R5011132016('HMAC_GOSTR3411_2012_512', TEST_KEY + add)
        self.assertTrue('invalid key size' in str(context.exception))

    def test_key_512(self):
        add = bytearray(32)
        test_hmac = gostcrypto.gosthmac.new('HMAC_GOSTR3411_2012_512', TEST_KEY + add)
        test_hmac.update(TEST_DATA)
        self.assertEqual(test_hmac.digest(), TEST_HMAC_512)

    def test_hmac_raises(self):
        with self.assertRaises(ValueError) as context:
            test_hmac = gostcrypto.gosthmac.R5011132016(None, TEST_KEY)
        self.assertTrue('unsupported mode' in str(context.exception))

    def test_hmac_double_update(self):
        test_hmac = gostcrypto.gosthmac.new('HMAC_GOSTR3411_2012_256', TEST_KEY)
        test_hmac.update(TEST_DATA)
        test_hmac.update(TEST_DATA)
        self.assertEqual(test_hmac.digest(), TEST_HMAC_256_DOUBLE_UPDATE)



"""The module that implements block encryption according to GOST 34.12-2015
   ('Kuznechik' and 'Magma').

   Author: Evgeny Drobotun (c) 2020
   License: MIT

"""

from struct import pack

from gostcrypto.utils import add_xor
from gostcrypto.utils import zero_fill
from gostcrypto.utils import S_BOX
from gostcrypto.utils import S_BOX_REVERSE

__all__ = ['GOST34122015Kuznechik', 'GOST34122015Magma']

_BLOCK_SIZE_KUZNECHIK = 16
_BLOCK_SIZE_MAGMA = 8
_KEY_SIZE = 32

#Constants for linear transformation in the 'Kuznechik' cipher
_L = bytearray([
    0x01, 0x94, 0x20, 0x85, 0x10, 0xc2, 0xc0, 0x01,
    0xfb, 0x01, 0xc0, 0xc2, 0x10, 0x85, 0x20, 0x94,
])

#Constants for nonlinear bijective transformation in the 'Magma' cipher
_S_BOX_MAGMA = (
    (0x0c, 0x04, 0x06, 0x02, 0x0a, 0x05, 0x0b, 0x09,
     0x0e, 0x08, 0x0d, 0x07, 0x00, 0x03, 0x0f, 0x01,),
    (0x06, 0x08, 0x02, 0x03, 0x09, 0x0a, 0x05, 0x0c,
     0x01, 0x0e, 0x04, 0x07, 0x0b, 0x0d, 0x00, 0x0f,),
    (0x0b, 0x03, 0x05, 0x08, 0x02, 0x0f, 0x0a, 0x0d,
     0x0e, 0x01, 0x07, 0x04, 0x0c, 0x09, 0x06, 0x00,),
    (0x0c, 0x08, 0x02, 0x01, 0x0d, 0x04, 0x0f, 0x06,
     0x07, 0x00, 0x0a, 0x05, 0x03, 0x0e, 0x09, 0x0b,),
    (0x07, 0x0f, 0x05, 0x0a, 0x08, 0x01, 0x06, 0x0d,
     0x00, 0x09, 0x03, 0x0e, 0x0b, 0x04, 0x02, 0x0c,),
    (0x05, 0x0d, 0x0f, 0x06, 0x09, 0x02, 0x0c, 0x0a,
     0x0b, 0x07, 0x08, 0x01, 0x04, 0x03, 0x0e, 0x00,),
    (0x08, 0x0e, 0x02, 0x05, 0x06, 0x09, 0x01, 0x0c,
     0x0f, 0x04, 0x0b, 0x00, 0x0d, 0x0a, 0x03, 0x07,),
    (0x01, 0x07, 0x0e, 0x0d, 0x00, 0x05, 0x08, 0x03,
     0x04, 0x0f, 0x0a, 0x06, 0x09, 0x0c, 0x0b, 0x02,)
)

class GOST34122015Kuznechik:
    """Class that implements block encryption in accordance with GOST 34.12-2015 ('Kuznechik').

       Methods:
          :decrypt(): decrypting a block of ciphertext.
          :encrypt(): encrypting a block of plaintext.
          :clear(): Сlearing the values of iterative encryption keys.

       Attributes:
          :block_size: an integer value the internal block size of the cipher algorithm in bytes.
          :key_size: an integer value the cipher key size.
    """
    def __init__(self, key):
        """Initialize the ciphering object.

           Args:
              :key: Cipher key.
        """
        self._cipher_c = []
        self._cipher_iter_key = []
        self._cipher_get_c()
        key_1 = key[:_KEY_SIZE // 2]
        key_2 = key[_KEY_SIZE // 2:]
        internal = bytearray(_KEY_SIZE // 2)
        self._cipher_iter_key.append(key_1)
        self._cipher_iter_key.append(key_2)
        for i in range(4):
            for j in range(8):
                internal = add_xor(key_1, self._cipher_c[i * 8 + j])
                internal = GOST34122015Kuznechik._cipher_s(internal)
                internal = GOST34122015Kuznechik._cipher_l(internal)
                key_1, key_2 = [add_xor(internal, key_2), key_1]
            self._cipher_iter_key.append(key_1)
            self._cipher_iter_key.append(key_2)
        key_1 = bytearray(self.key_size // 2)
        key_2 = bytearray(self.key_size // 2)
        key = bytearray(self.key_size)

    def __del__(self):
        #Delete the ciphering object
        self.clear()

    @staticmethod
    def _cipher_s(data):
        #The S-transformation function (nonlinear bijective transformation)
        result = bytearray(_BLOCK_SIZE_KUZNECHIK)
        for i in range(_BLOCK_SIZE_KUZNECHIK):
            result[i] = S_BOX[data[i]]
        return result

    @staticmethod
    def _cipher_s_reverse(data):
        #The reverse S-transformation function (reverse nonlinear bijective transformation)
        result = bytearray(_BLOCK_SIZE_KUZNECHIK)
        for i in range(_BLOCK_SIZE_KUZNECHIK):
            result[i] = S_BOX_REVERSE[data[i]]
        return result

    @staticmethod
    def _cipher_gf(op_a, op_b):
        #The GF-transformation function
        result = 0
        for _ in range(8):
            if op_b & 1:
                result = result ^ op_a
            hi_bit = op_a & 0x80
            op_a = op_a << 1
            if hi_bit:
                op_a = op_a ^ 0x1c3
            op_b = op_b >> 1
        return result

    @staticmethod
    def _cipher_r(data):
        #The R-transformation function
        a_0 = 0
        result = bytearray(_BLOCK_SIZE_KUZNECHIK)
        for i in range(_BLOCK_SIZE_KUZNECHIK):
            result[i] = data[i - 1]
            a_0 = a_0 ^ GOST34122015Kuznechik._cipher_gf(result[i], _L[i])
        result[0] = a_0
        return result

    @staticmethod
    def _cipher_r_reverse(data):
        #The reverse R-transformation function
        a_15 = 0
        result = bytearray(_BLOCK_SIZE_KUZNECHIK)
        for i in range(_BLOCK_SIZE_KUZNECHIK - 1, -1, -1):
            result[i - 1] = data[i]
            a_15 = a_15 ^ GOST34122015Kuznechik._cipher_gf(data[i], _L[i])
        result[15] = a_15
        return result

    @staticmethod
    def _cipher_l(data):
        #The L-transformation function (linear transformation)
        result = bytearray(_BLOCK_SIZE_KUZNECHIK)
        result = data
        for _ in range(16):
            result = GOST34122015Kuznechik._cipher_r(result)
        return result

    @staticmethod
    def _cipher_l_reverse(data):
        #The reverse L-transformation function (reverse linear transformation)
        result = bytearray(_BLOCK_SIZE_KUZNECHIK)
        result = data
        for _ in range(16):
            result = GOST34122015Kuznechik._cipher_r_reverse(result)
        return result

    def _cipher_get_c(self):
        #The generate iteration constants C
        for i in range(1, 33):
            internal = bytearray(_BLOCK_SIZE_KUZNECHIK)
            internal[15] = i
            self._cipher_c.append(GOST34122015Kuznechik._cipher_l(internal))

    @property
    def block_size(self):
        """An integer value the internal block size of the cipher algorithm in bytes.
           For the 'Kuznechik' algorithm this value is 16 and the 'Magma' algorithm,
           this value is 8.
        """
        return _BLOCK_SIZE_KUZNECHIK

    @property
    def key_size(self):
        """An integer value the cipher key size.
        """
        return _KEY_SIZE

    def decrypt(self, block):
        """Decrypting a block of ciphertext.

           Args:
             :block: The block of ciphertext to be decrypted (the block size is 16 bytes).

           Return:
              The block of plaintext.
        """
        block = bytearray(block)
        block = add_xor(self._cipher_iter_key[9], block)
        for i in range(8, -1, -1):
            block = GOST34122015Kuznechik._cipher_l_reverse(block)
            block = GOST34122015Kuznechik._cipher_s_reverse(block)
            block = add_xor(self._cipher_iter_key[i], block)
        return block

    def encrypt(self, block):
        """Encrypting a block of plaintext.

           Args:
             :block: The block of plaintext to be encrypted (the block size is 16 bytes).

           Return:
              The block of ciphertext.
        """
        block = bytearray(block)
        for i in range(9):
            block = add_xor(self._cipher_iter_key[i], block)
            block = GOST34122015Kuznechik._cipher_s(block)
            block = GOST34122015Kuznechik._cipher_l(block)
        block = add_xor(self._cipher_iter_key[9], block)
        return block

    def clear(self):
        """Сlearing the values of iterative encryption keys.
        """
        for i in range(10):
            self._cipher_iter_key[i] = zero_fill(len(self._cipher_iter_key[i]))

class GOST34122015Magma:
    """Class that implements block encryption in accordance with GOST 34.12-2012 ('Magma').

       Methods:
          :decrypt(): decrypting a block of ciphertext.
          :encrypt(): encrypting a block of plaintext.
          :clear(): Сlearing the values of iterative encryption keys.

       Attributes:
          :block_size: an integer value the internal block size of the cipher algorithm in bytes.
          :key_size: an integer value the cipher key size.
    """
    def __init__(self, key):
        """Initialize the ciphering object.

           Args:
              :key: Cipher key.
        """
        self._cipher_iter_key = []
        iter_key = b''
        for j in range(8):
            iter_key = bytearray(4)
            for i in range(4):
                iter_key[i] = key[(j * 4) + i]
            self._cipher_iter_key.append(iter_key)
        for j in range(8):
            iter_key = bytearray(4)
            for i in range(4):
                iter_key[i] = key[(j * 4) + i]
            self._cipher_iter_key.append(iter_key)
        for j in range(8):
            iter_key = bytearray(4)
            for i in range(4):
                iter_key[i] = key[(j * 4) + i]
            self._cipher_iter_key.append(iter_key)
        for j in range(8):
            iter_key = bytearray(4)
            for i in range(4):
                iter_key[i] = key[28 - (j * 4) + i]
            self._cipher_iter_key.append(iter_key)
        iter_key = zero_fill(len(iter_key))
        key = zero_fill(len(key))

    def __del__(self):
        self.clear()

    @staticmethod
    def _cipher_t(data):
        #The T-transformation function
        result = bytearray(4)
        data = bytearray(data)
        for i in range(4):
            first_part_byte = data[i] & 0x0f
            sec_part_byte = (data[i] & 0xf0) >> 4
            first_part_byte = _S_BOX_MAGMA[(3 - i) * 2][first_part_byte]
            sec_part_byte = _S_BOX_MAGMA[(3 - i) * 2 + 1][sec_part_byte]
            result[i] = (sec_part_byte << 4) | first_part_byte
        return result

    @staticmethod
    def _cipher_add_32(op_a, op_b):
        #Addition operation in the residue ring modulo 32
        op_a = bytearray(op_a)
        op_b = bytearray(op_b)
        internal = 0
        result = bytearray(4)
        for i in range(3, -1, -1):
            internal = op_a[i] + op_b[i] + (internal >> 8)
            result[i] = internal & 0xff
        return result

    @staticmethod
    def _cipher_g(cipher_k, cipher_a):
        #The g-transformation function
        cipher_k = bytearray(cipher_k)
        cipher_a = bytearray(cipher_a)
        internal = bytearray(4)
        result = bytearray(4)
        result_32 = 0
        internal = GOST34122015Magma._cipher_add_32(cipher_k, cipher_a)
        internal = GOST34122015Magma._cipher_t(internal)
        result_32 = internal[0]
        result_32 = (result_32 << 8) + internal[1]
        result_32 = (result_32 << 8) + internal[2]
        result_32 = (result_32 << 8) + internal[3]
        result_32 = (result_32 << 11)|(result_32 >> 21)
        result_32 = result_32 & 0xffffffff
        result = pack('>I', result_32)
        return result

    @staticmethod
    def _cipher_g_prev(cipher_k, cipher_a):
        #The preliminary G-transform function
        cipher_k = bytearray(cipher_k)
        cipher_a = bytearray(cipher_a)
        a_0 = bytearray(4)
        a_1 = bytearray(4)
        cipher_g = bytearray(4)
        result = bytearray(_BLOCK_SIZE_MAGMA)
        a_1 = cipher_a[0:4]
        a_0 = cipher_a[4:_BLOCK_SIZE_MAGMA]
        cipher_g = GOST34122015Magma._cipher_g(cipher_k, a_0)
        cipher_g = add_xor(a_1, cipher_g)
        a_1 = a_0
        a_0 = cipher_g
        result[0:4] = a_1
        result[4:_BLOCK_SIZE_MAGMA] = a_0
        return result

    @staticmethod
    def _cipher_g_fin(cipher_k, cipher_a):
        #The final G-transform function
        cipher_k = bytearray(cipher_k)
        cipher_a = bytearray(cipher_a)
        a_0 = bytearray(4)
        a_1 = bytearray(4)
        cipher_g = bytearray(4)
        result = bytearray(_BLOCK_SIZE_MAGMA)
        a_1 = cipher_a[0:4]
        a_0 = cipher_a[4:_BLOCK_SIZE_MAGMA]
        cipher_g = GOST34122015Magma._cipher_g(cipher_k, a_0)
        cipher_g = add_xor(a_1, cipher_g)
        a_1 = cipher_g
        result[0:4] = a_1
        result[4:_BLOCK_SIZE_MAGMA] = a_0
        return result

    @property
    def block_size(self):
        """An integer value the internal block size of the cipher algorithm in bytes.
           For the 'Kuznechik' algorithm this value is 16 and the 'Magma' algorithm,
           this value is 8.
        """
        return _BLOCK_SIZE_MAGMA

    @property
    def key_size(self):
        """An integer value the cipher key size.
        """
        return _KEY_SIZE

    def decrypt(self, block):
        """Decrypting a block of plaintext.

           Args:
             :block: The block of plaintext to be encrypted (the block size is 8 bytes).

           Return:
              The block of ciphertext.
        """
        result = bytearray(_BLOCK_SIZE_MAGMA)
        result = GOST34122015Magma._cipher_g_prev(self._cipher_iter_key[31], block)
        for i in range(30, 0, -1):
            result = GOST34122015Magma._cipher_g_prev(self._cipher_iter_key[i], result)
        result = GOST34122015Magma._cipher_g_fin(self._cipher_iter_key[0], result)
        return result

    def encrypt(self, block):
        """Encrypting a block of ciphertext.

           Args:
             :block: The block of ciphertext to be decrypted (the block size is 8 bytes).

           Return:
              The block of plaintext.
        """
        result = bytearray(_BLOCK_SIZE_MAGMA)
        result = GOST34122015Magma._cipher_g_prev(self._cipher_iter_key[0], block)
        for i in range(1, 31):
            result = GOST34122015Magma._cipher_g_prev(self._cipher_iter_key[i], result)
        result = GOST34122015Magma._cipher_g_fin(self._cipher_iter_key[31], result)
        return result

    def clear(self):
        """Сlearing the values of iterative encryption keys.
        """
        for i in range(32):
            self._cipher_iter_key[i] = zero_fill(len(self._cipher_iter_key[i]))



"""The module implementing the hash calculation algorithm GOST 34.11-2012 ('Streebog').

   Author: Evgeny Drobotun (c) 2020
   License: MIT

   Usage:
    - getting a hash for a string:

        import gostcrypto

        hasher = gostcrypto.gosthash.new('streebog256')
        hasher.update(b'<string>')
        result = hasher.hexdigest()

    - getting a hash for a file:

        import gostcrypto

        #The 'buffer_size' must be a multiple of 64
        buffer_size = 128
        hasher = gostcrypto.gosthash.new('streebog512')
        with open(<'file path'>, 'rb') as file:
            buffer = file.read(buffer_size)
            while len(buffer) > 0:
                hasher.update(buffer)
                buffer = file.read(buffer_size)
        result = hasher.hexdigest()

"""
from sys import exit as sys_exit
from copy import deepcopy
from struct import pack
from struct import unpack

from gostcrypto.utils import add_xor
from gostcrypto.utils import S_BOX

__all__ = ['GOST34112012', 'new']

_BLOCK_SIZE = 64

#Initialization vectors
_V_0 = bytearray(_BLOCK_SIZE)
_V_512 = bytearray([
    0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
])

# pylint: disable=bad-whitespace,bad-continuation
#Permutation matrix
_TAU = (
    0,  8, 16, 24, 32, 40, 48, 56,
    1,  9, 17, 25, 33, 41, 49, 57,
    2, 10, 18, 26, 34, 42, 50, 58,
    3, 11, 19, 27, 35, 43, 51, 59,
    4, 12, 20, 28, 36, 44, 52, 60,
    5, 13, 21, 29, 37, 45, 53, 61,
    6, 14, 22, 30, 38, 46, 54, 62,
    7, 15, 23, 31, 39, 47, 55, 63,
)
# pylint: disable=bad-whitespace,bad-continuation

#Matrix of linear transformation of a set of binary vectors
_A = (
    0x8e20faa72ba0b470, 0x47107ddd9b505a38,
    0xad08b0e0c3282d1c, 0xd8045870ef14980e,
    0x6c022c38f90a4c07, 0x3601161cf205268d,
    0x1b8e0b0e798c13c8, 0x83478b07b2468764,
    0xa011d380818e8f40, 0x5086e740ce47c920,
    0x2843fd2067adea10, 0x14aff010bdd87508,
    0x0ad97808d06cb404, 0x05e23c0468365a02,
    0x8c711e02341b2d01, 0x46b60f011a83988e,
    0x90dab52a387ae76f, 0x486dd4151c3dfdb9,
    0x24b86a840e90f0d2, 0x125c354207487869,
    0x092e94218d243cba, 0x8a174a9ec8121e5d,
    0x4585254f64090fa0, 0xaccc9ca9328a8950,
    0x9d4df05d5f661451, 0xc0a878a0a1330aa6,
    0x60543c50de970553, 0x302a1e286fc58ca7,
    0x18150f14b9ec46dd, 0x0c84890ad27623e0,
    0x0642ca05693b9f70, 0x0321658cba93c138,
    0x86275df09ce8aaa8, 0x439da0784e745554,
    0xafc0503c273aa42a, 0xd960281e9d1d5215,
    0xe230140fc0802984, 0x71180a8960409a42,
    0xb60c05ca30204d21, 0x5b068c651810a89e,
    0x456c34887a3805b9, 0xac361a443d1c8cd2,
    0x561b0d22900e4669, 0x2b838811480723ba,
    0x9bcf4486248d9f5d, 0xc3e9224312c8c1a0,
    0xeffa11af0964ee50, 0xf97d86d98a327728,
    0xe4fa2054a80b329c, 0x727d102a548b194e,
    0x39b008152acb8227, 0x9258048415eb419d,
    0x492c024284fbaec0, 0xaa16012142f35760,
    0x550b8e9e21f7a530, 0xa48b474f9ef5dc18,
    0x70a6a56e2440598e, 0x3853dc371220a247,
    0x1ca76e95091051ad, 0x0edd37c48a08a6d8,
    0x07e095624504536c, 0x8d70c431ac02a736,
    0xc83862965601dd1b, 0x641c314b2b8ee083,
)

#Iterative constants
_C  = [bytearray([
    0x07, 0x45, 0xa6, 0xf2, 0x59, 0x65, 0x80, 0xdd,
    0x23, 0x4d, 0x74, 0xcc, 0x36, 0x74, 0x76, 0x05,
    0x15, 0xd3, 0x60, 0xa4, 0x08, 0x2a, 0x42, 0xa2,
    0x01, 0x69, 0x67, 0x92, 0x91, 0xe0, 0x7c, 0x4b,
    0xfc, 0xc4, 0x85, 0x75, 0x8d, 0xb8, 0x4e, 0x71,
    0x16, 0xd0, 0x45, 0x2e, 0x43, 0x76, 0x6a, 0x2f,
    0x1f, 0x7c, 0x65, 0xc0, 0x81, 0x2f, 0xcb, 0xeb,
    0xe9, 0xda, 0xca, 0x1e, 0xda, 0x5b, 0x08, 0xb1
    ]), bytearray([
    0xb7, 0x9b, 0xb1, 0x21, 0x70, 0x04, 0x79, 0xe6,
    0x56, 0xcd, 0xcb, 0xd7, 0x1b, 0xa2, 0xdd, 0x55,
    0xca, 0xa7, 0x0a, 0xdb, 0xc2, 0x61, 0xb5, 0x5c,
    0x58, 0x99, 0xd6, 0x12, 0x6b, 0x17, 0xb5, 0x9a,
    0x31, 0x01, 0xb5, 0x16, 0x0f, 0x5e, 0xd5, 0x61,
    0x98, 0x2b, 0x23, 0x0a, 0x72, 0xea, 0xfe, 0xf3,
    0xd7, 0xb5, 0x70, 0x0f, 0x46, 0x9d, 0xe3, 0x4f,
    0x1a, 0x2f, 0x9d, 0xa9, 0x8a, 0xb5, 0xa3, 0x6f
    ]), bytearray([
    0xb2, 0x0a, 0xba, 0x0a, 0xf5, 0x96, 0x1e, 0x99,
    0x31, 0xdb, 0x7a, 0x86, 0x43, 0xf4, 0xb6, 0xc2,
    0x09, 0xdb, 0x62, 0x60, 0x37, 0x3a, 0xc9, 0xc1,
    0xb1, 0x9e, 0x35, 0x90, 0xe4, 0x0f, 0xe2, 0xd3,
    0x7b, 0x7b, 0x29, 0xb1, 0x14, 0x75, 0xea, 0xf2,
    0x8b, 0x1f, 0x9c, 0x52, 0x5f, 0x5e, 0xf1, 0x06,
    0x35, 0x84, 0x3d, 0x6a, 0x28, 0xfc, 0x39, 0x0a,
    0xc7, 0x2f, 0xce, 0x2b, 0xac, 0xdc, 0x74, 0xf5
    ]), bytearray([
    0x2e, 0xd1, 0xe3, 0x84, 0xbc, 0xbe, 0x0c, 0x22,
    0xf1, 0x37, 0xe8, 0x93, 0xa1, 0xea, 0x53, 0x34,
    0xbe, 0x03, 0x52, 0x93, 0x33, 0x13, 0xb7, 0xd8,
    0x75, 0xd6, 0x03, 0xed, 0x82, 0x2c, 0xd7, 0xa9,
    0x3f, 0x35, 0x5e, 0x68, 0xad, 0x1c, 0x72, 0x9d,
    0x7d, 0x3c, 0x5c, 0x33, 0x7e, 0x85, 0x8e, 0x48,
    0xdd, 0xe4, 0x71, 0x5d, 0xa0, 0xe1, 0x48, 0xf9,
    0xd2, 0x66, 0x15, 0xe8, 0xb3, 0xdf, 0x1f, 0xef
    ]), bytearray([
    0x57, 0xfe, 0x6c, 0x7c, 0xfd, 0x58, 0x17, 0x60,
    0xf5, 0x63, 0xea, 0xa9, 0x7e, 0xa2, 0x56, 0x7a,
    0x16, 0x1a, 0x27, 0x23, 0xb7, 0x00, 0xff, 0xdf,
    0xa3, 0xf5, 0x3a, 0x25, 0x47, 0x17, 0xcd, 0xbf,
    0xbd, 0xff, 0x0f, 0x80, 0xd7, 0x35, 0x9e, 0x35,
    0x4a, 0x10, 0x86, 0x16, 0x1f, 0x1c, 0x15, 0x7f,
    0x63, 0x23, 0xa9, 0x6c, 0x0c, 0x41, 0x3f, 0x9a,
    0x99, 0x47, 0x47, 0xad, 0xac, 0x6b, 0xea, 0x4b
    ]), bytearray([
    0x6e, 0x7d, 0x64, 0x46, 0x7a, 0x40, 0x68, 0xfa,
    0x35, 0x4f, 0x90, 0x36, 0x72, 0xc5, 0x71, 0xbf,
    0xb6, 0xc6, 0xbe, 0xc2, 0x66, 0x1f, 0xf2, 0x0a,
    0xb4, 0xb7, 0x9a, 0x1c, 0xb7, 0xa6, 0xfa, 0xcf,
    0xc6, 0x8e, 0xf0, 0x9a, 0xb4, 0x9a, 0x7f, 0x18,
    0x6c, 0xa4, 0x42, 0x51, 0xf9, 0xc4, 0x66, 0x2d,
    0xc0, 0x39, 0x30, 0x7a, 0x3b, 0xc3, 0xa4, 0x6f,
    0xd9, 0xd3, 0x3a, 0x1d, 0xae, 0xae, 0x4f, 0xae
    ]), bytearray([
    0x93, 0xd4, 0x14, 0x3a, 0x4d, 0x56, 0x86, 0x88,
    0xf3, 0x4a, 0x3c, 0xa2, 0x4c, 0x45, 0x17, 0x35,
    0x04, 0x05, 0x4a, 0x28, 0x83, 0x69, 0x47, 0x06,
    0x37, 0x2c, 0x82, 0x2d, 0xc5, 0xab, 0x92, 0x09,
    0xc9, 0x93, 0x7a, 0x19, 0x33, 0x3e, 0x47, 0xd3,
    0xc9, 0x87, 0xbf, 0xe6, 0xc7, 0xc6, 0x9e, 0x39,
    0x54, 0x09, 0x24, 0xbf, 0xfe, 0x86, 0xac, 0x51,
    0xec, 0xc5, 0xaa, 0xee, 0x16, 0x0e, 0xc7, 0xf4
    ]), bytearray([
    0x1e, 0xe7, 0x02, 0xbf, 0xd4, 0x0d, 0x7f, 0xa4,
    0xd9, 0xa8, 0x51, 0x59, 0x35, 0xc2, 0xac, 0x36,
    0x2f, 0xc4, 0xa5, 0xd1, 0x2b, 0x8d, 0xd1, 0x69,
    0x90, 0x06, 0x9b, 0x92, 0xcb, 0x2b, 0x89, 0xf4,
    0x9a, 0xc4, 0xdb, 0x4d, 0x3b, 0x44, 0xb4, 0x89,
    0x1e, 0xde, 0x36, 0x9c, 0x71, 0xf8, 0xb7, 0x4e,
    0x41, 0x41, 0x6e, 0x0c, 0x02, 0xaa, 0xe7, 0x03,
    0xa7, 0xc9, 0x93, 0x4d, 0x42, 0x5b, 0x1f, 0x9b
    ]), bytearray([
    0xdb, 0x5a, 0x23, 0x83, 0x51, 0x44, 0x61, 0x72,
    0x60, 0x2a, 0x1f, 0xcb, 0x92, 0xdc, 0x38, 0x0e,
    0x54, 0x9c, 0x07, 0xa6, 0x9a, 0x8a, 0x2b, 0x7b,
    0xb1, 0xce, 0xb2, 0xdb, 0x0b, 0x44, 0x0a, 0x80,
    0x84, 0x09, 0x0d, 0xe0, 0xb7, 0x55, 0xd9, 0x3c,
    0x24, 0x42, 0x89, 0x25, 0x1b, 0x3a, 0x7d, 0x3a,
    0xde, 0x5f, 0x16, 0xec, 0xd8, 0x9a, 0x4c, 0x94,
    0x9b, 0x22, 0x31, 0x16, 0x54, 0x5a, 0x8f, 0x37
    ]), bytearray([
    0xed, 0x9c, 0x45, 0x98, 0xfb, 0xc7, 0xb4, 0x74,
    0xc3, 0xb6, 0x3b, 0x15, 0xd1, 0xfa, 0x98, 0x36,
    0xf4, 0x52, 0x76, 0x3b, 0x30, 0x6c, 0x1e, 0x7a,
    0x4b, 0x33, 0x69, 0xaf, 0x02, 0x67, 0xe7, 0x9f,
    0x03, 0x61, 0x33, 0x1b, 0x8a, 0xe1, 0xff, 0x1f,
    0xdb, 0x78, 0x8a, 0xff, 0x1c, 0xe7, 0x41, 0x89,
    0xf3, 0xf3, 0xe4, 0xb2, 0x48, 0xe5, 0x2a, 0x38,
    0x52, 0x6f, 0x05, 0x80, 0xa6, 0xde, 0xbe, 0xab
    ]), bytearray([
    0x1b, 0x2d, 0xf3, 0x81, 0xcd, 0xa4, 0xca, 0x6b,
    0x5d, 0xd8, 0x6f, 0xc0, 0x4a, 0x59, 0xa2, 0xde,
    0x98, 0x6e, 0x47, 0x7d, 0x1d, 0xcd, 0xba, 0xef,
    0xca, 0xb9, 0x48, 0xea, 0xef, 0x71, 0x1d, 0x8a,
    0x79, 0x66, 0x84, 0x14, 0x21, 0x80, 0x01, 0x20,
    0x61, 0x07, 0xab, 0xeb, 0xbb, 0x6b, 0xfa, 0xd8,
    0x94, 0xfe, 0x5a, 0x63, 0xcd, 0xc6, 0x02, 0x30,
    0xfb, 0x89, 0xc8, 0xef, 0xd0, 0x9e, 0xcd, 0x7b
    ]), bytearray([
    0x20, 0xd7, 0x1b, 0xf1, 0x4a, 0x92, 0xbc, 0x48,
    0x99, 0x1b, 0xb2, 0xd9, 0xd5, 0x17, 0xf4, 0xfa,
    0x52, 0x28, 0xe1, 0x88, 0xaa, 0xa4, 0x1d, 0xe7,
    0x86, 0xcc, 0x91, 0x18, 0x9d, 0xef, 0x80, 0x5d,
    0x9b, 0x9f, 0x21, 0x30, 0xd4, 0x12, 0x20, 0xf8,
    0x77, 0x1d, 0xdf, 0xbc, 0x32, 0x3c, 0xa4, 0xcd,
    0x7a, 0xb1, 0x49, 0x04, 0xb0, 0x80, 0x13, 0xd2,
    0xba, 0x31, 0x16, 0xf1, 0x67, 0xe7, 0x8e, 0x37
    ])
]

def new(name):
    """Creates a new hashing object and returns it.

       Args:
          :name: The string with the name of the hashing algorithm ('streebog256'
             for the GOST R 34.11-2012 algorithm with the resulting hash length
             of 32 bytes or 'streebog512' with the resulting hash length of 64 bytes.

       Return:
          New hashing object.

       Exception:
          ValueError('unsupported hash type') - in case of invalid value 'name'.
    """
    try:
        return GOST34112012(name)
    except ValueError as err:
        print(err)
        sys_exit()

class GOST34112012:
    """Class that implements the hash calculation algorithm GOST 34.11-2012 ('Streebog').

       Methods:
          :update(): update the hash object with the bytes-like object.
          :copy(): returns a copy ('clone') of the hash object.
          :digest(): returns the digest of the data passed to the 'update()' method so far.
          :hexdigest(): returns a digest of the hexadecimal data passed so far to the 'update()'
             method.
          :reset(): resets the values of all class attributes.

       Attributes:
          :digest_size: an integer value the size of the resulting hash in bytes.
          :block_size: an integer value the internal block size of the hash algorithm in bytes.
          :name: a text string value the name of the hashing algorithm.
    """

    def __init__(self, name):
        #Initialize the hashing object
        if name not in ('streebog512', 'streebog256'):
            raise ValueError('ValueError: unsupported hash type')
        self._name = name
        self._buff = b''
        self._num_block = 0
        self._pad_block_size = 0
        self._hash_h = bytearray(_BLOCK_SIZE)
        self._hash_n = bytearray(_BLOCK_SIZE)
        self._hash_sigma = bytearray(_BLOCK_SIZE)
        if self._name == 'streebog256':
            self._hash_h = _BLOCK_SIZE * b'\x01'

    @staticmethod
    def _hash_add_512(op_a, op_b):
        #Addition operation in the residue ring modulo 512
        op_a = bytearray(op_a)
        op_b = bytearray(op_b)
        op_c = 0
        result = bytearray(_BLOCK_SIZE)
        for i in range(_BLOCK_SIZE):
            op_c = op_a[i] + op_b[i] + (op_c >> 8)
            result[i] = op_c & 0xff
        return result

    @staticmethod
    def _hash_p(data):
        #The operation of the permutation byte
        result = bytearray(_BLOCK_SIZE)
        for i in range(_BLOCK_SIZE - 1, -1, -1):
            result[i] = data[_TAU[i]]
        return result

    @staticmethod
    def _hash_s(data):
        ##The S-transformation function (nonlinear bijective transformation)
        result = bytearray(_BLOCK_SIZE)
        for i in range(_BLOCK_SIZE - 1, -1, -1):
            result[i] = S_BOX[data[i]]
        return result

    @staticmethod
    def _hash_l(data):
        #Linear transformation of a set of binary vectors
        result = []
        for i in range(8):
            internal = unpack('<Q', data[i * 8:i * 8 + 8])[0]
            result_64 = 0
            for j in range(_BLOCK_SIZE - 1, -1, -1):
                if internal & 1:
                    result_64 = result_64 ^ _A[j]
                internal = internal >> 1
            result.append(pack('<Q', result_64))
        return b"".join(result)

    def _hash_get_key(self, k, i):
        #The function of the generation iterative compression keys
        key = bytearray(_BLOCK_SIZE)
        key = add_xor(k, _C[i])
        key = self._hash_s(key)
        key = self._hash_p(key)
        key = self._hash_l(key)
        return key

    def _hash_e(self, k, data):
        #The E-transformation function
        internal = add_xor(k, data)
        for i in range(12):
            internal = self._hash_s(internal)
            internal = self._hash_p(internal)
            internal = self._hash_l(internal)
            k = self._hash_get_key(k, i)
            internal = add_xor(internal, k)
        return internal

    def _hash_g(self, hash_h, hash_n, data):
        #The compression function
        k = add_xor(hash_n, hash_h)
        k = self._hash_s(k)
        k = self._hash_p(k)
        k = self._hash_l(k)

        internal = self._hash_e(k, data)
        internal = add_xor(internal, hash_h)
        result = add_xor(internal, data)
        return result

    def update(self, data):
        """Update the hash object with the bytes-like object.

           Args:
             :data: The string from which to get the hash. Repeated calls are equivalent
                to a single call with the concatenation of all the arguments: 'm.update(a)';
                'm.update(b)' is equivalent to 'm.update(a+b)'.
        """
        data = self._buff + data
        self._num_block = len(data) // _BLOCK_SIZE
        for i in range(0, self._num_block * _BLOCK_SIZE, _BLOCK_SIZE):
            block = data[i:i + _BLOCK_SIZE]
            self._hash_h = self._hash_g(self._hash_h, self._hash_n, block)
            self._hash_n = self._hash_add_512(self._hash_n, _V_512)
            self._hash_sigma = self._hash_add_512(self._hash_sigma, block)
        self._pad_block_size = _BLOCK_SIZE - len(data) % _BLOCK_SIZE
        if self._pad_block_size < _BLOCK_SIZE:
            self._buff = data[-(_BLOCK_SIZE - self._pad_block_size):]

    def hash_final(self):
        """Completes the hash calculation after the data update.
        """
        self._pad_block_size = _BLOCK_SIZE - len(self._buff)
        internal = bytearray(_BLOCK_SIZE)
        internal[1] = (((_BLOCK_SIZE - self._pad_block_size) * 8) >> 8) & 0xff
        internal[0] = ((_BLOCK_SIZE - self._pad_block_size) * 8) & 0xff
        if self._pad_block_size <= _BLOCK_SIZE:
            self._buff = self._buff + b'\x01'
            self._buff = self._buff + b'\x00' * (self._pad_block_size - 1)

        self._hash_h = self._hash_g(self._hash_h, self._hash_n, self._buff)
        self._hash_n = self._hash_add_512(self._hash_n, internal)
        self._hash_sigma = self._hash_add_512(self._hash_sigma, self._buff)
        self._hash_h = self._hash_g(self._hash_h, _V_0, self._hash_n)
        self._hash_h = self._hash_g(self._hash_h, _V_0, self._hash_sigma)

    def get_hash(self):
        """Returns the value of the _hasha_h attribute.
        """
        return self._hash_h[-self.digest_size:]

    def digest(self):
        """Returns the digest of the data passed to the update() method so far.
           This is a bytes object of size digest_size.
        """
        temp = self.copy()
        temp.hash_final()
        return temp.get_hash()

    def hexdigest(self):
        """Returns the digest of the data passed to the update() method so far.
           This is a double-sized string object (digest_size * 2).
        """
        return self.digest().hex()

    def reset(self):
        """Resets the values of all class attributes.
        """
        self._buff = b''
        self._num_block = 0
        self._pad_block_size = 0
        self._hash_h = bytearray(_BLOCK_SIZE)
        self._hash_n = bytearray(_BLOCK_SIZE)
        self._hash_sigma = bytearray(_BLOCK_SIZE)
        if self._name == 'streebog256':
            self._hash_h = _BLOCK_SIZE * b'\x01'

    def copy(self):
        """Returns a copy (“clone”) of the hash object. This can be used to efficiently compute
           the digests of data sharing a common initial substring.
        """
        return deepcopy(self)

    @property
    def digest_size(self):
        """An integer value the size of the resulting hash in bytes.
           For the 'streebog256' algorithm, this value is 32, for the
           'streebog512' algorithm, this value is 64.
        """
        if self._name == 'streebog512':
            result = 64
        else:
            result = 32
        return result

    @property
    def block_size(self):
        """An integer value the internal block size of the hash algorithm in bytes.
           For the 'streebog256' algorithm and the 'streebog512' algorithm,
           this value is 64.
        """
        return _BLOCK_SIZE

    @property
    def name(self):
        """A text string value the name of the hashing algorithm.
           Respectively 'streebog256' or 'streebog512'.
        """
        return self._name



"""The module that implements auxiliary functions for the operation of the
   GOSTcrypto module.
"""

__title__ = 'utils'
__author__ = 'Evgeny Drobotun'
__author_email__ = 'drobotun@xakep.ru'
__license__ = 'MIT'
__copyright__ = 'Copyright (C) 2020 Evgeny Drobotun'

from .utils import *
from .s_box import *



"""The module implementing the calculating the HMAC message authentication code in
   accordance with R 50.1.113-2016.

   Author: Evgeny Drobotun (c) 2020
   License: MIT

   Usage:
    - getting a HMAC for a string:

        import gostcrypto

        hmac_obj = gostcrypto.gosthmac.new('HMAC_GOSTR3411_2012_256')
        hmac_obj.update(b'<string>')
        result = hmac_obj.hexdigest()

    - getting a HMAC for a file:

        import gostcrypto

        #The 'buffer_size' must be a multiple of 64
        buffer_size = 128
        hmac_obj = gostcrypto.gosthash.new('HMAC_GOSTR3411_2012_256')
        with open(<'file path'>, 'rb') as file:
            buffer = file.read(buffer_size)
            while len(buffer) > 0:
                hmac_obj.update(buffer)
                buffer = file.read(buffer_size)
        result = hmac_obj.hexdigest()

"""
from sys import exit as sys_exit
from copy import deepcopy

from gostcrypto.gosthash import GOST34112012
from gostcrypto.utils import zero_fill
from gostcrypto.utils import add_xor

__all__ = ['R5011132016', 'new']

_KEY_SIZE = 64

#Сonstant 'ipad' (in accordance with RFC 2104)
_I_PAD = bytearray([
    0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36,
    0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36,
    0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36,
    0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36,
    0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36,
    0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36,
    0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36,
    0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36, 0x36,
])

#Сonstant 'opad' (in accordance with RFC 2104)
_O_PAD = bytearray([
    0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c,
    0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c,
    0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c,
    0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c,
    0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c,
    0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c,
    0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c,
    0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c, 0x5c,
])

def new(name, key):
    """Creates a new authentication code calculation object and returns it.

       Args:
          :name: Name of the authentication code calculation mode ('HMAC_GOSTR3411_2012_256'
             or 'HMAC_GOSTR3411_2012_512').
          :key: Authentication key.

       Return:
          New authentication code calculation object.

       Exception:
          ValueError('unsupported mode') - in case of unsupported mode.
          ValueError('invalid key size') - in case of invalid key size.
    """
    try:
        return R5011132016(name, key)
    except ValueError as err:
        print(err)
        sys_exit()

class R5011132016:
    """Class that implementing the calculating the HMAC message authentication code in
       accordance with R 50.1.113-2016.

       Methods:
          :update(): update the HMAC object with the bytes-like object.
          :digest(): getting the authentication code.
          :clear(): clears the key value.
          :copy(): returns a copy (“clone”) of the HMAC object.

       Attributes:
          :digest_size: an integer value of the size of the resulting HMAC digest in bytes.
          :block_size: an integer value the internal block size of the hash algorithm in bytes.
          :name: a text string is the name of the authentication code calculation algorithm.
    """

    def __init__(self, name, key):
        #Initialize the HMAC object
        if name not in ('HMAC_GOSTR3411_2012_256', 'HMAC_GOSTR3411_2012_512'):
            raise ValueError('ValueError: unsupported mode')
        if len(key) < _KEY_SIZE:
            add = bytearray(_KEY_SIZE - len(key))
            self._key = key + add
            self._key = bytearray(self._key)
        elif len(key) == _KEY_SIZE:
            self._key = key
        else:
            key = zero_fill(len(key))
            raise ValueError('ValueError: invalid key size')
        key = bytearray(len(key))
        if name == 'HMAC_GOSTR3411_2012_256':
            self._hasher_obj = GOST34112012('streebog256')
        elif name == 'HMAC_GOSTR3411_2012_512':
            self._hasher_obj = GOST34112012('streebog512')
        self._counter = 0

    def __del__(self):
        #Delete the HMAC object
        if hasattr(self, '_hasher_obj'):
            self.clear()

    def update(self, data):
        """Update the HMAC object with the bytes-like object.

           Args:
             :data: The message for which want to calculate the authentication code. Repeated
                calls are equivalent to a single call with the concatenation of all the
                arguments: m.update(a); m.update(b) is equivalent to m.update(a+b).
        """
        self._counter = self._counter + 1
        if self._counter == 1:
            self._hasher_obj.update(add_xor(self._key, _I_PAD) + data)
        elif self._counter != 1:
            self._hasher_obj.update(data)

    def digest(self):
        """Returns the HMAC message authentication code.
        """
        fin_hasher_obj = GOST34112012(self._hasher_obj.name)
        fin_hasher_obj.update(add_xor(self._key, _O_PAD) + self._hasher_obj.digest())
        result = fin_hasher_obj.digest()
        fin_hasher_obj.reset()
        return result

    def hexdigest(self):
        """Returns the HMAC message authentication code as a hexadecimal string.
        """
        fin_hasher_obj = GOST34112012(self._hasher_obj.name)
        fin_hasher_obj.update(add_xor(self._key, _O_PAD) + self._hasher_obj.digest())
        result = fin_hasher_obj.digest()
        fin_hasher_obj.reset()
        return result.hex()

    def copy(self):
        """Returns a copy (“clone”) of the HMAC object. This can be used to efficiently compute
           the digests of data sharing a common initial substring.
        """
        return deepcopy(self)

    def reset(self):
        """Resets the values of all class attributes.
        """
        self._hasher_obj.reset()
        self._counter = 0

    def clear(self):
        """Сlears the key value.
        """
        self._hasher_obj.reset()
        self._key = zero_fill(_KEY_SIZE)

    @property
    def digest_size(self):
        """An integer value of the size of the resulting HMAC digest in bytes.
        """
        return self._hasher_obj.digest_size

    @property
    def block_size(self):
        """An integer value the internal block size of the hash algorithm in bytes.
           For the 'streebog256' algorithm and the 'streebog512' algorithm,
           this value is 64.
        """
        return self._hasher_obj.block_size

    @property
    def name(self):
        """A text string is the name of the authentication code calculation algorithm.
        """
        if self._hasher_obj.name == 'streebog256':
            result = 'HMAC_GOSTR3411_2012_256'
        else:
            result = 'HMAC_GOSTR3411_2012_512'
        return result



"""The module that implements pseudo-random sequence generation in accordance with
   R 1323565.1.006-2017.
"""

__title__ = 'gostrandom'
__author__ = 'Evgeny Drobotun'
__author_email__ = 'drobotun@xakep.ru'
__license__ = 'MIT'
__copyright__ = 'Copyright (C) 2020 Evgeny Drobotun'

from .r_1323565_1_006_2017 import *



import unittest
import gostcrypto

class Test(unittest.TestCase):
    
    TEST_MSG_SHORT = bytearray([
        0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,
        0x38, 0x39, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35,
        0x36, 0x37, 0x38, 0x39, 0x30, 0x31, 0x32, 0x33,
        0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x30, 0x31,
        0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39,
        0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,
        0x38, 0x39, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35,
        0x36, 0x37, 0x38, 0x39, 0x30, 0x31, 0x32
        ])

    TEST_MSG_LONG = u'Се ветри, Стрибожи внуци, веютъ с моря стрелами на храбрыя плъкы Игоревы'.encode('cp1251')

    def test_hash_add_512(self):
        test_hasher = gostcrypto.gosthash.new('streebog256')
        op_a = self.TEST_MSG_SHORT + b'\x01'
        op_b = test_hasher.block_size * b'\xaf'
        test_result = ('dfe0e1e2e3e4e5e6e7e8dfe0e1e2e3e4e5e6e7e8dfe0e1e2e3e4e5e6e7e8dfe0e1e2e3e4e5e6e7e8dfe0e1e2e3e4e5e6e7e8dfe0e1e2e3e4e5e6e7e8dfe0e1b0')
        result = gostcrypto.gosthash.GOST34112012._hash_add_512(op_a, op_b)
        self.assertEqual(''.join(format(x, '02x') for x in result), test_result)

    def test_hash_p(self):
        test_hasher = gostcrypto.gosthash.new('streebog256')
        test_msg = self.TEST_MSG_SHORT + b'\x01'
        test_result = ('30383634323038363139373533313937323038363432303833313937353331393432303836343230353331393735333136343230383634323735333139373501')
        result = test_hasher._hash_p(test_msg)
        self.assertEqual(''.join(format(x, '02x') for x in result), test_result)

    def test_hash_l(self):
        test_hasher = gostcrypto.gosthash.new('streebog256')
        test_msg = self.TEST_MSG_SHORT + b'\x01'
        test_result = ('5e2a7862ae04af3627e702c039be3e73b9f661a6d680831976840907c18e8c907380b2ad67509bc15e2a7862ae04af3627e702c039be3e736350d1f348630177')
        result = test_hasher._hash_l(test_msg)
        self.assertEqual(''.join(format(x, '02x') for x in result), test_result)

    def test_hash_s(self):
        test_hasher = gostcrypto.gosthash.new('streebog256')
        test_msg = self.TEST_MSG_SHORT + b'\x01'
        test_result = ('058402aee36a8fa0060b058402aee36a8fa0060b058402aee36a8fa0060b058402aee36a8fa0060b058402aee36a8fa0060b058402aee36a8fa0060b058402ee')
        result = test_hasher._hash_s(test_msg)
        self.assertEqual(''.join(format(x, '02x') for x in result), test_result)

    def test_hash_get_key(self):
        test_hasher = gostcrypto.gosthash.new('streebog256')
        test_msg = self.TEST_MSG_SHORT + b'\x01'
        test_result = ('6d897244067f1c66de4deaa188aaf436db4af5cae20e688bd5f8848aa13dcbd54f07aadbc4869a662b70a3082d5701b5fc78715327ec5fc5898136507ff251d2')
        result = test_hasher._hash_get_key(test_msg, 1)
        self.assertEqual(''.join(format(x, '02x') for x in result), test_result)

    def test_hash_e(self):
        test_hasher = gostcrypto.gosthash.new('streebog256')
        test_msg = self.TEST_MSG_SHORT + b'\x01'
        test_result = ('44c0a9aeea44c9852d4434191482da6c7eabac88b279e82526ec6f61580caca61939f4cbff1d4c9aa8518313d393271009b38fedf1c8690ce3556bbb04e82f49')
        result = test_hasher._hash_e(test_msg, test_msg)
        self.assertEqual(''.join(format(x, '02x') for x in result), test_result)

    def test_hash_g(self):
        test_hasher = gostcrypto.gosthash.new('streebog256')
        test_msg = self.TEST_MSG_SHORT + b'\x01'
        test_h = test_hasher.block_size * b'\x00'
        test_n = test_hasher.block_size * b'\x00'
        test_result = ('e2da3b6b73e4fe05d9f5b13f793541955c81502c520fedd3c5babb8c90f65427bd8e7333db8a4826a6a95a444166a817384f3921af34ea9111cb2c81f82c10fd')
        result = test_hasher._hash_g(test_h, test_n, test_msg)
        self.assertEqual(''.join(format(x, '02x') for x in result), test_result)

    def test_new(self):
        test_hasher = gostcrypto.gosthash.new('streebog256')
        _test_hasher = gostcrypto.gosthash.GOST34112012('streebog256')
        self.assertEqual(test_hasher._name, _test_hasher._name)
        self.assertEqual(test_hasher._buff, _test_hasher._buff)
        self.assertEqual(test_hasher._num_block, _test_hasher._num_block)
        self.assertEqual(test_hasher._pad_block_size, _test_hasher._pad_block_size)
        self.assertEqual(test_hasher._hash_h, _test_hasher._hash_h)
        self.assertEqual(test_hasher._hash_n, _test_hasher._hash_n)
        self.assertEqual(test_hasher._hash_sigma, _test_hasher._hash_sigma)

    def test_init_raises(self):
        with self.assertRaises(ValueError) as context:
            test_hasher =  gostcrypto.gosthash.GOST34112012('test_name')
        self.assertTrue('unsupported hash type' in str(context.exception))

    def test_digest_size(self):
        test_hasher = gostcrypto.gosthash.new('streebog256')
        self.assertEqual(test_hasher.digest_size, 32)
        test_hasher = gostcrypto.gosthash.new('streebog512')
        self.assertEqual(test_hasher.digest_size, 64)

    def test_block_size(self):
        test_hasher = gostcrypto.gosthash.new('streebog256')
        self.assertEqual(test_hasher.block_size, 64)
        test_hasher = gostcrypto.gosthash.new('streebog512')
        self.assertEqual(test_hasher.block_size, 64)

    def test_name(self):
        test_hasher = gostcrypto.gosthash.new('streebog256')
        result = test_hasher.name
        self.assertEqual(result, 'streebog256')

    def test_update(self):
        test_hasher = gostcrypto.gosthash.new('streebog512')
        test_hasher.update(self.TEST_MSG_LONG)
        test_result = 'afc707c4da6c812e85ea2c8e1e225d2e481f7389883b84a76aa4c7f79e77ef987b510c9b45acb727e1f8108f934e91e25d399597cd4cbbe365a4fa1223607fcd'
        self.assertEqual(''.join(format(x, '02x') for x in test_hasher._hash_h), test_result)

    def test_reset_512(self):
        test_hasher = gostcrypto.gosthash.new('streebog512')
        test_hasher.update(self.TEST_MSG_LONG)
        test_hasher.digest()
        test_hasher.reset()
        self.assertEqual(test_hasher._hash_h, bytearray(test_hasher.block_size))
        self.assertEqual(test_hasher._hash_n, bytearray(test_hasher.block_size))
        self.assertEqual(test_hasher._hash_sigma, bytearray(test_hasher.block_size))
        self.assertEqual(test_hasher._num_block, 0)
        self.assertEqual(test_hasher._pad_block_size, 0)

    def test_reset_256(self):
        test_hasher = gostcrypto.gosthash.new('streebog256')
        test_hasher.update(self.TEST_MSG_LONG)
        test_hasher.digest()
        test_hasher.reset()
        self.assertEqual(test_hasher._hash_h, test_hasher.block_size * b'\x01')
        self.assertEqual(test_hasher._hash_n, bytearray(test_hasher.block_size))
        self.assertEqual(test_hasher._hash_sigma, bytearray(test_hasher.block_size))
        self.assertEqual(test_hasher._num_block, 0)
        self.assertEqual(test_hasher._pad_block_size, 0)

    def test_digest(self):
        test_hasher = gostcrypto.gosthash.new('streebog512')
        test_hasher.update(self.TEST_MSG_LONG)
        result = test_hasher.digest()
        test_result = '1e88e62226bfca6f9994f1f2d51569e0daf8475a3b0fe61a5300eee46d961376035fe83549ada2b8620fcd7c496ce5b33f0cb9dddc2b6460143b03dabac9fb28'
        self.assertEqual(''.join(format(x, '02x') for x in result), test_result)

    def test_digest_double_update(self):
        test_hasher = gostcrypto.gosthash.new('streebog512')
        test_hasher.update(u'Се ветри, Стрибожи внуци, веютъ с моря '.encode('cp1251'))
        test_hasher.digest()
        test_hasher.update(u'стрелами на храбрыя плъкы Игоревы'.encode('cp1251'))
        result = test_hasher.digest()
        test_result = '1e88e62226bfca6f9994f1f2d51569e0daf8475a3b0fe61a5300eee46d961376035fe83549ada2b8620fcd7c496ce5b33f0cb9dddc2b6460143b03dabac9fb28'
        self.assertEqual(''.join(format(x, '02x') for x in result), test_result)

    def test_hexdigest(self):
        test_hasher = gostcrypto.gosthash.new('streebog512')
        test_hasher.update(self.TEST_MSG_LONG)
        result = test_hasher.hexdigest()
        test_result = '1e88e62226bfca6f9994f1f2d51569e0daf8475a3b0fe61a5300eee46d961376035fe83549ada2b8620fcd7c496ce5b33f0cb9dddc2b6460143b03dabac9fb28'
        self.assertEqual(result, test_result)

    def test_habr_144(self):
        test_hasher = gostcrypto.gosthash.new('streebog256')
        test_hasher.update(bytearray.fromhex('d0cf11e0a1b11ae1000000000000000000000000000000003e000300feff0900060000000000000000000000010000000100\
                                              000000000000001000002400000001000000feffffff0000000000000000ffffffffffffffffffffffffffffffffffffffff\
                                              ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff'))
        result = test_hasher.hexdigest()
        test_result = 'c766085540caaa8953bfcf7a1ba220619cee50d65dc242f82f23ba4b180b18e0'
        self.assertEqual(result, test_result)

if __name__ == '__main__':
    unittest.main()




import unittest
import gostcrypto

class TestKuznechik(unittest.TestCase):

    TEST_KEY = bytearray([
        0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
        0xfe, 0xdc, 0xba, 0x98, 0x76, 0x54, 0x32, 0x10, 0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef,
    ])

    ENCRYPT_TEST_STRING = bytearray([
        0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x00, 0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88,
    ])

    DECRYPT_TEST_STRING = bytearray([
        0x7f, 0x67, 0x9d, 0x90, 0xbe, 0xbc, 0x24, 0x30, 0x5a, 0x46, 0x8d, 0x42, 0xb9, 0xd4, 0xed, 0xcd,
    ])

    def test_s(self):
        test_data_1 = bytearray([
            0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x00,
        ])
        test_data_2 = bytearray([
            0xb6, 0x6c, 0xd8, 0x88, 0x7d, 0x38, 0xe8, 0xd7, 0x77, 0x65, 0xae, 0xea, 0x0c, 0x9a, 0x7e, 0xfc,
        ])
        test_data_3 = bytearray([
            0x55, 0x9d, 0x8d, 0xd7, 0xbd, 0x06, 0xcb, 0xfe, 0x7e, 0x7b, 0x26, 0x25, 0x23, 0x28, 0x0d, 0x39,
        ])
        test_data_4 = bytearray([
            0x0c, 0x33, 0x22, 0xfe, 0xd5, 0x31, 0xe4, 0x63, 0x0d, 0x80, 0xef, 0x5c, 0x5a, 0x81, 0xc5, 0x0b,
        ])
        test_data_5 = bytearray([
            0x23, 0xae, 0x65, 0x63, 0x3f, 0x84, 0x2d, 0x29, 0xc5, 0xdf, 0x52, 0x9c, 0x13, 0xf5, 0xac, 0xda,
        ])
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_s(test_data_1), test_data_2)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_s(test_data_2), test_data_3)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_s(test_data_3), test_data_4)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_s(test_data_4), test_data_5)

    def test_s_reverse(self):
        test_data_1 = bytearray([
            0x23, 0xae, 0x65, 0x63, 0x3f, 0x84, 0x2d, 0x29, 0xc5, 0xdf, 0x52, 0x9c, 0x13, 0xf5, 0xac, 0xda,
        ])
        test_data_2 = bytearray([
            0x0c, 0x33, 0x22, 0xfe, 0xd5, 0x31, 0xe4, 0x63, 0x0d, 0x80, 0xef, 0x5c, 0x5a, 0x81, 0xc5, 0x0b,
        ])
        test_data_3 = bytearray([
            0x55, 0x9d, 0x8d, 0xd7, 0xbd, 0x06, 0xcb, 0xfe, 0x7e, 0x7b, 0x26, 0x25, 0x23, 0x28, 0x0d, 0x39,
        ])
        test_data_4 = bytearray([
            0xb6, 0x6c, 0xd8, 0x88, 0x7d, 0x38, 0xe8, 0xd7, 0x77, 0x65, 0xae, 0xea, 0x0c, 0x9a, 0x7e, 0xfc,
        ])
        test_data_5 = bytearray([
            0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x00,
        ])
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_s_reverse(test_data_1), test_data_2)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_s_reverse(test_data_2), test_data_3)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_s_reverse(test_data_3), test_data_4)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_s_reverse(test_data_4), test_data_5)

    def test_r(self):
        test_data_1 = bytearray([
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00,
        ])
        test_data_2 = bytearray([
            0x94, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
        ])
        test_data_3 = bytearray([
            0xa5, 0x94, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        ])
        test_data_4 = bytearray([
            0x64, 0xa5, 0x94, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        ])
        test_data_5 = bytearray([
            0x0d, 0x64, 0xa5, 0x94, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        ])
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_r(test_data_1), test_data_2)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_r(test_data_2), test_data_3)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_r(test_data_3), test_data_4)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_r(test_data_4), test_data_5)

    def test_r_reverse(self):
        test_data_1 = bytearray([
            0x0d, 0x64, 0xa5, 0x94, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        ])
        test_data_2 = bytearray([
            0x64, 0xa5, 0x94, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        ])
        test_data_3 = bytearray([
            0xa5, 0x94, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        ])
        test_data_4 = bytearray([
            0x94, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
        ])
        test_data_5 = bytearray([
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00,
        ])
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_r_reverse(test_data_1), test_data_2)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_r_reverse(test_data_2), test_data_3)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_r_reverse(test_data_3), test_data_4)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_r_reverse(test_data_4), test_data_5)

    def test_l(self):
        test_data_1 = bytearray([
            0x64, 0xa5, 0x94, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        ])
        test_data_2 = bytearray([
            0xd4, 0x56, 0x58, 0x4d, 0xd0, 0xe3, 0xe8, 0x4c, 0xc3, 0x16, 0x6e, 0x4b, 0x7f, 0xa2, 0x89, 0x0d,
        ])
        test_data_3 = bytearray([
            0x79, 0xd2, 0x62, 0x21, 0xb8, 0x7b, 0x58, 0x4c, 0xd4, 0x2f, 0xbc, 0x4f, 0xfe, 0xa5, 0xde, 0x9a,
        ])
        test_data_4 = bytearray([
            0x0e, 0x93, 0x69, 0x1a, 0x0c, 0xfc, 0x60, 0x40, 0x8b, 0x7b, 0x68, 0xf6, 0x6b, 0x51, 0x3c, 0x13,
        ])
        test_data_5 = bytearray([
            0xe6, 0xa8, 0x09, 0x4f, 0xee, 0x0a, 0xa2, 0x04, 0xfd, 0x97, 0xbc, 0xb0, 0xb4, 0x4b, 0x85, 0x80,
        ])
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_l(test_data_1), test_data_2)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_l(test_data_2), test_data_3)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_l(test_data_3), test_data_4)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_l(test_data_4), test_data_5)

    def test_l_reverse(self):
        test_data_1 = bytearray([
            0xe6, 0xa8, 0x09, 0x4f, 0xee, 0x0a, 0xa2, 0x04, 0xfd, 0x97, 0xbc, 0xb0, 0xb4, 0x4b, 0x85, 0x80,
        ])
        test_data_2 = bytearray([
            0x0e, 0x93, 0x69, 0x1a, 0x0c, 0xfc, 0x60, 0x40, 0x8b, 0x7b, 0x68, 0xf6, 0x6b, 0x51, 0x3c, 0x13,
        ])
        test_data_3 = bytearray([
            0x79, 0xd2, 0x62, 0x21, 0xb8, 0x7b, 0x58, 0x4c, 0xd4, 0x2f, 0xbc, 0x4f, 0xfe, 0xa5, 0xde, 0x9a,
        ])
        test_data_4 = bytearray([
            0xd4, 0x56, 0x58, 0x4d, 0xd0, 0xe3, 0xe8, 0x4c, 0xc3, 0x16, 0x6e, 0x4b, 0x7f, 0xa2, 0x89, 0x0d,
        ])
        test_data_5 = bytearray([
            0x64, 0xa5, 0x94, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        ])
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_l_reverse(test_data_1), test_data_2)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_l_reverse(test_data_2), test_data_3)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_l_reverse(test_data_3), test_data_4)
        self.assertEqual(gostcrypto.GOST34122015Kuznechik._cipher_l_reverse(test_data_4), test_data_5)

    def test_expand_key(self):
        test_iter_key = [bytearray([
            0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
            ]), bytearray([
            0xfe, 0xdc, 0xba, 0x98, 0x76, 0x54, 0x32, 0x10, 0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef,
            ]), bytearray([
            0xdb, 0x31, 0x48, 0x53, 0x15, 0x69, 0x43, 0x43, 0x22, 0x8d, 0x6a, 0xef, 0x8c, 0xc7, 0x8c, 0x44,
            ]), bytearray([
            0x3d, 0x45, 0x53, 0xd8, 0xe9, 0xcf, 0xec, 0x68, 0x15, 0xeb, 0xad, 0xc4, 0x0a, 0x9f, 0xfd, 0x04,
            ]), bytearray([
            0x57, 0x64, 0x64, 0x68, 0xc4, 0x4a, 0x5e, 0x28, 0xd3, 0xe5, 0x92, 0x46, 0xf4, 0x29, 0xf1, 0xac,
            ]), bytearray([
            0xbd, 0x07, 0x94, 0x35, 0x16, 0x5c, 0x64, 0x32, 0xb5, 0x32, 0xe8, 0x28, 0x34, 0xda, 0x58, 0x1b,
            ]), bytearray([
            0x51, 0xe6, 0x40, 0x75, 0x7e, 0x87, 0x45, 0xde, 0x70, 0x57, 0x27, 0x26, 0x5a, 0x00, 0x98, 0xb1,
            ]), bytearray([
            0x5a, 0x79, 0x25, 0x01, 0x7b, 0x9f, 0xdd, 0x3e, 0xd7, 0x2a, 0x91, 0xa2, 0x22, 0x86, 0xf9, 0x84,
            ]), bytearray([
            0xbb, 0x44, 0xe2, 0x53, 0x78, 0xc7, 0x31, 0x23, 0xa5, 0xf3, 0x2f, 0x73, 0xcd, 0xb6, 0xe5, 0x17,
            ]), bytearray([
            0x72, 0xe9, 0xdd, 0x74, 0x16, 0xbc, 0xf4, 0x5b, 0x75, 0x5d, 0xba, 0xa8, 0x8e, 0x4a, 0x40, 0x43,
            ])
        ]
        test_cipher = gostcrypto.gostcipher.GOST34122015Kuznechik(self.TEST_KEY)
        self.assertEqual(test_cipher._cipher_iter_key[0], test_iter_key[0])
        self.assertEqual(test_cipher._cipher_iter_key[1], test_iter_key[1])
        self.assertEqual(test_cipher._cipher_iter_key[2], test_iter_key[2])
        self.assertEqual(test_cipher._cipher_iter_key[3], test_iter_key[3])
        self.assertEqual(test_cipher._cipher_iter_key[4], test_iter_key[4])
        self.assertEqual(test_cipher._cipher_iter_key[5], test_iter_key[5])
        self.assertEqual(test_cipher._cipher_iter_key[6], test_iter_key[6])
        self.assertEqual(test_cipher._cipher_iter_key[7], test_iter_key[7])
        self.assertEqual(test_cipher._cipher_iter_key[8], test_iter_key[8])
        self.assertEqual(test_cipher._cipher_iter_key[9], test_iter_key[9])

    def test_encrypt(self):
        test_cipher = gostcrypto.gostcipher.GOST34122015Kuznechik(self.TEST_KEY)
        self.assertEqual(test_cipher.encrypt(self.ENCRYPT_TEST_STRING), self.DECRYPT_TEST_STRING)

    def test_decrypt(self):
        test_cipher = gostcrypto.gostcipher.GOST34122015Kuznechik(self.TEST_KEY)
        self.assertEqual(test_cipher.decrypt(self.DECRYPT_TEST_STRING), self.ENCRYPT_TEST_STRING)

    def test_key_size(self):
        test_cipher = gostcrypto.gostcipher.GOST34122015Kuznechik(self.TEST_KEY)
        self.assertEqual(test_cipher.key_size, 32)

class TestMagma(unittest.TestCase):

    TEST_KEY = bytearray([
        0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88,
        0x77, 0x66, 0x55, 0x44, 0x33, 0x22, 0x11, 0x00,
        0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7,
        0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
    ])

    ENCRYPT_TEST_STRING = bytearray([
        0xfe, 0xdc, 0xba, 0x98, 0x76, 0x54, 0x32, 0x10,
    ])

    DECRYPT_TEST_STRING = bytearray([
        0x4e, 0xe9, 0x01, 0xe5, 0xc2, 0xd8, 0xca, 0x3d,
    ])

    def test_t(self):
        test_data_1 = bytearray([
            0xfd, 0xb9, 0x75, 0x31,
        ])
        test_data_2 = bytearray([
            0x2a, 0x19, 0x6f, 0x34,
        ])
        test_data_3 = bytearray([
            0xeb, 0xd9, 0xf0, 0x3a,
        ])
        test_data_4 = bytearray([
            0xb0, 0x39, 0xbb, 0x3d,
        ])
        test_data_5 = bytearray([
            0x68, 0x69, 0x54, 0x33,
        ])
        self.assertEqual(gostcrypto.GOST34122015Magma._cipher_t(test_data_1), test_data_2)
        self.assertEqual(gostcrypto.GOST34122015Magma._cipher_t(test_data_2), test_data_3)
        self.assertEqual(gostcrypto.GOST34122015Magma._cipher_t(test_data_3), test_data_4)
        self.assertEqual(gostcrypto.GOST34122015Magma._cipher_t(test_data_4), test_data_5)

    def test_encrypt(self):
        test_cipher = gostcrypto.gostcipher.GOST34122015Magma(self.TEST_KEY)
        self.assertEqual(test_cipher.encrypt(self.ENCRYPT_TEST_STRING), self.DECRYPT_TEST_STRING)

    def test_decrypt(self):
        test_cipher = gostcrypto.gostcipher.GOST34122015Magma(self.TEST_KEY)
        self.assertEqual(test_cipher.decrypt(self.DECRYPT_TEST_STRING), self.ENCRYPT_TEST_STRING)

    def test_key_size(self):
        test_cipher = gostcrypto.gostcipher.GOST34122015Magma(self.TEST_KEY)
        self.assertEqual(test_cipher.key_size, 32)

class TestGOST34132015Kuznechik(unittest.TestCase):

    TEST_KEY = bytearray([
        0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xdd, 0xee, 0xff, 0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
        0xfe, 0xdc, 0xba, 0x98, 0x76, 0x54, 0x32, 0x10, 0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef,
    ])

    TEST_PLAIN_TEXT = bytearray([
        0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x00, 0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88,
        0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xee, 0xff, 0x0a,
        0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xee, 0xff, 0x0a, 0x00,
        0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xee, 0xff, 0x0a, 0x00, 0x11,
    ])

    TEST_INIT_VECT = bytearray([
        0x12, 0x34, 0x56, 0x78, 0x90, 0xab, 0xce, 0xf0, 0xa1, 0xb2, 0xc3, 0xd4, 0xe5, 0xf0, 0x01, 0x12,
        0x23, 0x34, 0x45, 0x56, 0x67, 0x78, 0x89, 0x90, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19,
    ])

    TEST_INIT_VECT_CTR = bytearray([
        0x12, 0x34, 0x56, 0x78, 0x90, 0xab, 0xce, 0xf0,
    ])

    TEST_CIPHER_TEXT_ECB = bytearray([
        0x7f, 0x67, 0x9d, 0x90, 0xbe, 0xbc, 0x24, 0x30, 0x5a, 0x46, 0x8d, 0x42, 0xb9, 0xd4, 0xed, 0xcd,
        0xb4, 0x29, 0x91, 0x2c, 0x6e, 0x00, 0x32, 0xf9, 0x28, 0x54, 0x52, 0xd7, 0x67, 0x18, 0xd0, 0x8b,
        0xf0, 0xca, 0x33, 0x54, 0x9d, 0x24, 0x7c, 0xee, 0xf3, 0xf5, 0xa5, 0x31, 0x3b, 0xd4, 0xb1, 0x57,
        0xd0, 0xb0, 0x9c, 0xcd, 0xe8, 0x30, 0xb9, 0xeb, 0x3a, 0x02, 0xc4, 0xc5, 0xaa, 0x8a, 0xda, 0x98,
    ])

    TEST_CIPHER_TEXT_CBC = bytearray([
        0x68, 0x99, 0x72, 0xd4, 0xa0, 0x85, 0xfa, 0x4d, 0x90, 0xe5, 0x2e, 0x3d, 0x6d, 0x7d, 0xcc, 0x27,
        0x28, 0x26, 0xe6, 0x61, 0xb4, 0x78, 0xec, 0xa6, 0xaf, 0x1e, 0x8e, 0x44, 0x8d, 0x5e, 0xa5, 0xac,
        0xfe, 0x7b, 0xab, 0xf1, 0xe9, 0x19, 0x99, 0xe8, 0x56, 0x40, 0xe8, 0xb0, 0xf4, 0x9d, 0x90, 0xd0,
        0x16, 0x76, 0x88, 0x06, 0x5a, 0x89, 0x5c, 0x63, 0x1a, 0x2d, 0x9a, 0x15, 0x60, 0xb6, 0x39, 0x70,
    ])

    TEST_CIPHER_TEXT_CFB = bytearray([
        0x81, 0x80, 0x0a, 0x59, 0xb1, 0x84, 0x2b, 0x24, 0xff, 0x1f, 0x79, 0x5e, 0x89, 0x7a, 0xbd, 0x95,
        0xed, 0x5b, 0x47, 0xa7, 0x04, 0x8c, 0xfa, 0xb4, 0x8f, 0xb5, 0x21, 0x36, 0x9d, 0x93, 0x26, 0xbf,
        0x79, 0xf2, 0xa8, 0xeb, 0x5c, 0xc6, 0x8d, 0x38, 0x84, 0x2d, 0x26, 0x4e, 0x97, 0xa2, 0x38, 0xb5,
        0x4f, 0xfe, 0xbe, 0xcd, 0x4e, 0x92, 0x2d, 0xe6, 0xc7, 0x5b, 0xd9, 0xdd, 0x44, 0xfb, 0xf4, 0xd1,
    ])

    TEST_CIPHER_TEXT_OFB = bytearray([
        0x81, 0x80, 0x0a, 0x59, 0xb1, 0x84, 0x2b, 0x24, 0xff, 0x1f, 0x79, 0x5e, 0x89, 0x7a, 0xbd, 0x95,
        0xed, 0x5b, 0x47, 0xa7, 0x04, 0x8c, 0xfa, 0xb4, 0x8f, 0xb5, 0x21, 0x36, 0x9d, 0x93, 0x26, 0xbf,
        0x66, 0xa2, 0x57, 0xac, 0x3c, 0xa0, 0xb8, 0xb1, 0xc8, 0x0f, 0xe7, 0xfc, 0x10, 0x28, 0x8a, 0x13,
        0x20, 0x3e, 0xbb, 0xc0, 0x66, 0x13, 0x86, 0x60, 0xa0, 0x29, 0x22, 0x43, 0xf6, 0x90, 0x31, 0x50,
    ])

    TEST_CIPHER_TEXT_CTR = bytearray([
        0xf1, 0x95, 0xd8, 0xbe, 0xc1, 0x0e, 0xd1, 0xdb, 0xd5, 0x7b, 0x5f, 0xa2, 0x40, 0xbd, 0xa1, 0xb8,
        0x85, 0xee, 0xe7, 0x33, 0xf6, 0xa1, 0x3e, 0x5d, 0xf3, 0x3c, 0xe4, 0xb3, 0x3c, 0x45, 0xde, 0xe4,
        0xa5, 0xea, 0xe8, 0x8b, 0xe6, 0x35, 0x6e, 0xd3, 0xd5, 0xe8, 0x77, 0xf1, 0x35, 0x64, 0xa3, 0xa5,
        0xcb, 0x91, 0xfa, 0xb1, 0xf2, 0x0c, 0xba, 0xb6, 0xd1, 0xc6, 0xd1, 0x58, 0x20, 0xbd, 0xba, 0x73,
    ])

    TEST_PLAIN_TEXT_NO_MUL = bytearray([
        0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x00, 0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88,
        0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xee, 0xff, 0x0a,
        0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xee, 0xff, 0x0a, 0x00,
        0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xee, 0xff,
    ])

    TEST_PLAIN_TEXT_PAD_1 = bytearray([
        0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x00, 0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88,
        0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xee, 0xff, 0x0a,
        0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xee, 0xff, 0x0a, 0x00,
        0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xee, 0xff, 0x00, 0x00, 0x00,
    ])

    TEST_PLAIN_TEXT_PAD_2 = bytearray([
        0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x00, 0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88,
        0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xee, 0xff, 0x0a,
        0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xee, 0xff, 0x0a, 0x00,
        0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88, 0x99, 0xaa, 0xbb, 0xcc, 0xee, 0xff, 0x80, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    ])

    TEST_PLAIN_TEXT_SHORT = bytearray([
        0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x00, 0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa,
    ])

    TEST_CIPHER_TEXT_ECB_SHORT = bytearray([
        0x3c, 0xc3, 0x42, 0xdf, 0xcd, 0x5a, 0x7b, 0xef, 0x4e, 0x80, 0x1c, 0xa9, 0x6d, 0x84, 0x6f, 0xe6,
    ])

    TEST_PLAIN_TEXT_SHORT_PAD_1 = bytearray([
        0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x00, 0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x00, 0x00,
    ])

    TEST_CIPHER_TEXT_ECB_PAD_1 = bytearray([
        0x7f, 0x67, 0x9d, 0x90, 0xbe, 0xbc, 0x24, 0x30, 0x5a, 0x46, 0x8d, 0x42, 0xb9, 0xd4, 0xed, 0xcd,
        0xb4, 0x29, 0x91, 0x2c, 0x6e, 0x00, 0x32, 0xf9, 0x28, 0x54, 0x52, 0xd7, 0x67, 0x18, 0xd0, 0x8b,
        0xf0, 0xca, 0x33, 0x54, 0x9d, 0x24, 0x7c, 0xee, 0xf3, 0xf5, 0xa5, 0x31, 0x3b, 0xd4, 0xb1, 0x57,
        0xd8, 0x8c, 0xdb, 0x75, 0x60, 0x99, 0xf9, 0xd8, 0xd6, 0x57, 0x89, 0xb2, 0xce, 0x51, 0x42, 0x5c,
    ])

    TEST_CIPHER_TEXT_ECB_PAD_2 = bytearray([
        0x7f, 0x67, 0x9d, 0x90, 0xbe, 0xbc, 0x24, 0x30, 0x5a, 0x46, 0x8d, 0x42, 0xb9, 0xd4, 0xed, 0xcd,
        0xb4, 0x29, 0x91, 0x2c, 0x6e, 0x00, 0x32, 0xf9, 0x28, 0x54, 0x52, 0xd7, 0x67, 0x18, 0xd0, 0x8b,
        0xf0, 0xca, 0x33, 0x54, 0x9d, 0x24, 0x7c, 0xee, 0xf3, 0xf5, 0xa5, 0x31, 0x3b, 0xd4, 0xb1, 0x57,
        0xe2, 0xb7, 0x75, 0x12, 0x7f, 0x93, 0x2e, 0xce, 0x26, 0x2a, 0x60, 0x25, 0xc0, 0x63, 0xdb, 0xcb,
        0x94, 0xbe, 0xc1, 0x5e, 0x26, 0x9c, 0xf1, 0xe5, 0x06, 0xf0, 0x2b, 0x99, 0x4c, 0x0a, 0x8e, 0xa0

    ])

    TEST_CIPHER_TEXT_CTR_NO_MUL = bytearray([
        0xf1, 0x95, 0xd8, 0xbe, 0xc1, 0x0e, 0xd1, 0xdb, 0xd5, 0x7b, 0x5f, 0xa2, 0x40, 0xbd, 0xa1, 0xb8,
        0x85, 0xee, 0xe7, 0x33, 0xf6, 0xa1, 0x3e, 0x5d, 0xf3, 0x3c, 0xe4, 0xb3, 0x3c, 0x45, 0xde, 0xe4,
        0xa5, 0xea, 0xe8, 0x8b, 0xe6, 0x35, 0x6e, 0xd3, 0xd5, 0xe8, 0x77, 0xf1, 0x35, 0x64, 0xa3, 0xa5,
        0xcb, 0x91, 0xfa, 0xb1, 0xf2, 0x0c, 0xba, 0xb6, 0xd1, 0xc6, 0xd1, 0x58, 0x20,
    ])

    TEST_CIPHER_TEXT_CBC_PAD_1 = bytearray([
        0x68, 0x99, 0x72, 0xd4, 0xa0, 0x85, 0xfa, 0x4d, 0x90, 0xe5, 0x2e, 0x3d, 0x6d, 0x7d, 0xcc, 0x27,
        0x28, 0x26, 0xe6, 0x61, 0xb4, 0x78, 0xec, 0xa6, 0xaf, 0x1e, 0x8e, 0x44, 0x8d, 0x5e, 0xa5, 0xac,
        0xfe, 0x7b, 0xab, 0xf1, 0xe9, 0x19, 0x99, 0xe8, 0x56, 0x40, 0xe8, 0xb0, 0xf4, 0x9d, 0x90, 0xd0,
        0xe2, 0x8b, 0xb9, 0x02, 0x9c, 0x31, 0xaa, 0x5f, 0xe5, 0x56, 0x11, 0x92, 0xeb, 0x9c, 0x3f, 0x70,
    ])

    TEST_CIPHER_TEXT_CBC_PAD_2 = bytearray([
        0x68, 0x99, 0x72, 0xd4, 0xa0, 0x85, 0xfa, 0x4d, 0x90, 0xe5, 0x2e, 0x3d, 0x6d, 0x7d, 0xcc, 0x27,
        0x28, 0x26, 0xe6, 0x61, 0xb4, 0x78, 0xec, 0xa6, 0xaf, 0x1e, 0x8e, 0x44, 0x8d, 0x5e, 0xa5, 0xac,
        0xfe, 0x7b, 0xab, 0xf1, 0xe9, 0x19, 0x99, 0xe8, 0x56, 0x40, 0xe8, 0xb0, 0xf4, 0x9d, 0x90, 0xd0,
        0x26, 0x12, 0x5a, 0x8d, 0xde, 0x53, 0x5f, 0x61, 0x47, 0xdb, 0xde, 0xba, 0xd2, 0x83, 0x24, 0xbe,
        0x82, 0x8e, 0xa5, 0xda, 0xf2, 0x9f, 0x23, 0x9f, 0x47, 0x32, 0xcd, 0x08, 0xbd, 0xf6, 0xab, 0x18,
    ])

    TEST_CIPHER_TEXT_OFB_NO_MUL = bytearray([
        0x81, 0x80, 0x0a, 0x59, 0xb1, 0x84, 0x2b, 0x24, 0xff, 0x1f, 0x79, 0x5e, 0x89, 0x7a, 0xbd, 0x95,
        0xed, 0x5b, 0x47, 0xa7, 0x04, 0x8c, 0xfa, 0xb4, 0x8f, 0xb5, 0x21, 0x36, 0x9d, 0x93, 0x26, 0xbf,
        0x66, 0xa2, 0x57, 0xac, 0x3c, 0xa0, 0xb8, 0xb1, 0xc8, 0x0f, 0xe7, 0xfc, 0x10, 0x28, 0x8a, 0x13,
        0x20, 0x3e, 0xbb, 0xc0, 0x66, 0x13, 0x86, 0x60, 0xa0, 0x29, 0x22, 0x43, 0xf6,
    ])

    TEST_CIPHER_TEXT_CFB_NO_MUL = bytearray([
        0x81, 0x80, 0x0a, 0x59, 0xb1, 0x84, 0x2b, 0x24, 0xff, 0x1f, 0x79, 0x5e, 0x89, 0x7a, 0xbd, 0x95,
        0xed, 0x5b, 0x47, 0xa7, 0x04, 0x8c, 0xfa, 0xb4, 0x8f, 0xb5, 0x21, 0x36, 0x9d, 0x93, 0x26, 0xbf,
        0x79, 0xf2, 0xa8, 0xeb, 0x5c, 0xc6, 0x8d, 0x38, 0x84, 0x2d, 0x26, 0x4e, 0x97, 0xa2, 0x38, 0xb5,
        0x4f, 0xfe, 0xbe, 0xcd, 0x4e, 0x92, 0x2d, 0xe6, 0xc7, 0x5b, 0xd9, 0xdd, 0x44,
    ])

    

    TEST_MAC_VALUE = bytearray([
        0x33, 0x6f, 0x4d, 0x29, 0x60, 0x59, 0xfb, 0xe3, 0x4d, 0xde, 0xb3, 0x5b, 0x37, 0x74, 0x9c, 0x67,
    ])

    TEST_MAC_VALUE_DOUBLE = bytearray([
        0x83, 0xd2, 0x11, 0x9c, 0x63, 0xa3, 0x7f, 0x8e, 0x85, 0x62, 0x4d, 0x90, 0x0a, 0x25, 0x94, 0xb4,
    ])

    TEST_MAC_VALUE_PAD = bytearray([
        0x7e, 0xd1, 0xa2, 0xb9, 0x96, 0x48, 0x06, 0x04, 0xaa, 0xf5, 0x51, 0x80, 0x66, 0x5a, 0xdf, 0xc0,
    ])

    def test_init_raises(self):
        with self.assertRaises(ValueError) as context:
            test_obj =  gostcrypto.gostcipher.GOST34132015('test_algorithm', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB)
        self.assertTrue('unsupported algorithm' in str(context.exception))
        with self.assertRaises(ValueError) as context:
            test_obj =  gostcrypto.gostcipher.GOST34132015('kuznechik', 'test_key', None, pad_mode=None)
        self.assertTrue('invalid key value' in str(context.exception))
        with self.assertRaises(ValueError) as context:
            test_obj =  gostcrypto.gostcipher.GOST34132015('kuznechik', b'test_key', None, pad_mode=None)
        self.assertTrue('invalid key size' in str(context.exception))
        with self.assertRaises(ValueError) as context:
            test_obj =  gostcrypto.gostcipher.GOST34132015('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=None, pad_mode=None)
        self.assertTrue('invalid IV value' in str(context.exception))

    def test_ecb_encrypt(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT), self.TEST_CIPHER_TEXT_ECB)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT_NO_MUL), self.TEST_CIPHER_TEXT_ECB_PAD_1)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB, pad_mode=gostcrypto.gostcipher.PAD_MODE_2)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT_NO_MUL), self.TEST_CIPHER_TEXT_ECB_PAD_2)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT_SHORT), self.TEST_CIPHER_TEXT_ECB_SHORT)
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB, pad_mode=gostcrypto.gostcipher.PAD_MODE_3)
        with self.assertRaises(ValueError) as context:
            test_obj.encrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('unsupported padding mode' in str(context.exception))

    def test_ecb_decrypt(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB)
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_ECB), self.TEST_PLAIN_TEXT)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB)
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_ECB_PAD_1), self.TEST_PLAIN_TEXT_PAD_1)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB, pad_mode=gostcrypto.gostcipher.PAD_MODE_2)
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_ECB_PAD_2), self.TEST_PLAIN_TEXT_PAD_2)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB)
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_ECB_SHORT), self.TEST_PLAIN_TEXT_SHORT_PAD_1)
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB, pad_mode=gostcrypto.gostcipher.PAD_MODE_3)
        with self.assertRaises(ValueError) as context:
            test_obj.decrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('unsupported padding mode' in str(context.exception))

    def test_ctr_encrypt(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=self.TEST_INIT_VECT_CTR)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT), self.TEST_CIPHER_TEXT_CTR)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=self.TEST_INIT_VECT_CTR)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT_NO_MUL), self.TEST_CIPHER_TEXT_CTR_NO_MUL)
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=b'test_iv')
        with self.assertRaises(ValueError) as context:
            test_obj.encrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('invalid initialization vector size' in str(context.exception))

    def test_ctr_decrypt(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=self.TEST_INIT_VECT_CTR)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_CTR), self.TEST_PLAIN_TEXT)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=self.TEST_INIT_VECT_CTR)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_CTR_NO_MUL), self.TEST_PLAIN_TEXT_NO_MUL)
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=b'test_iv')
        with self.assertRaises(ValueError) as context:
            test_obj.decrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('invalid initialization vector size' in str(context.exception))

    def test_cbc_encrypt(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CBC, init_vect=self.TEST_INIT_VECT)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT), self.TEST_CIPHER_TEXT_CBC)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CBC, init_vect=self.TEST_INIT_VECT)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT_NO_MUL), self.TEST_CIPHER_TEXT_CBC_PAD_1)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CBC,
                                             init_vect=self.TEST_INIT_VECT, pad_mode=gostcrypto.gostcipher.PAD_MODE_2)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT_NO_MUL), self.TEST_CIPHER_TEXT_CBC_PAD_2)
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CBC, init_vect=self.TEST_INIT_VECT,
                                              pad_mode=gostcrypto.gostcipher.PAD_MODE_3)
        with self.assertRaises(ValueError) as context:
            test_obj.encrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('unsupported padding mode' in str(context.exception))
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CBC, init_vect=b'test_iv',
                                              pad_mode=gostcrypto.gostcipher.PAD_MODE_2)
        with self.assertRaises(ValueError) as context:
            test_obj.encrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('invalid initialization vector size' in str(context.exception))

    def test_cbc_decrypt(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CBC, init_vect=self.TEST_INIT_VECT)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_CBC), self.TEST_PLAIN_TEXT)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CBC, init_vect=self.TEST_INIT_VECT)
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_CBC_PAD_1), self.TEST_PLAIN_TEXT_PAD_1)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CBC,
                                             init_vect=self.TEST_INIT_VECT, pad_mode=gostcrypto.gostcipher.PAD_MODE_2)
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_CBC_PAD_2), self.TEST_PLAIN_TEXT_PAD_2)
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CBC,
                                              init_vect=self.TEST_INIT_VECT, pad_mode=gostcrypto.gostcipher.PAD_MODE_3)
        with self.assertRaises(ValueError) as context:
            test_obj.decrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('unsupported padding mode' in str(context.exception))
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CBC, init_vect=b'test_iv',
                                              pad_mode=gostcrypto.gostcipher.PAD_MODE_2)
        with self.assertRaises(ValueError) as context:
            test_obj.decrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('invalid initialization vector size' in str(context.exception))

    def test_cfb_encrypt(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CFB, init_vect=self.TEST_INIT_VECT)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT), self.TEST_CIPHER_TEXT_CFB)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CFB, init_vect=self.TEST_INIT_VECT)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT_NO_MUL), self.TEST_CIPHER_TEXT_CFB_NO_MUL)
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CFB, init_vect=b'test_iv')
        with self.assertRaises(ValueError) as context:
            test_obj.encrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('invalid initialization vector size' in str(context.exception))

    def test_cfb_decrypt(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CFB, init_vect=self.TEST_INIT_VECT)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_CFB), self.TEST_PLAIN_TEXT)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CFB, init_vect=self.TEST_INIT_VECT)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_CFB_NO_MUL), self.TEST_PLAIN_TEXT_NO_MUL)
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CFB, init_vect=b'test_iv')
        with self.assertRaises(ValueError) as context:
            test_obj.decrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('invalid initialization vector size' in str(context.exception))

    def test_ofb_encrypt(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_OFB, init_vect=self.TEST_INIT_VECT)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT), self.TEST_CIPHER_TEXT_OFB)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_OFB, init_vect=self.TEST_INIT_VECT)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT_NO_MUL), self.TEST_CIPHER_TEXT_OFB_NO_MUL)
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_OFB, init_vect=b'test_iv')
        with self.assertRaises(ValueError) as context:
            test_obj.encrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('invalid initialization vector size' in str(context.exception))

    def test_ofb_decrypt(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_OFB, init_vect=self.TEST_INIT_VECT)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_OFB), self.TEST_PLAIN_TEXT)
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_OFB, init_vect=self.TEST_INIT_VECT)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_OFB_NO_MUL), self.TEST_PLAIN_TEXT_NO_MUL)
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_OFB, init_vect=b'test_iv')
        with self.assertRaises(ValueError) as context:
            test_obj.decrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('invalid initialization vector size' in str(context.exception))

    def test_mac_calculate(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_MAC,
                                             pad_mode=gostcrypto.gostcipher.PAD_MODE_3)
        test_obj.update(self.TEST_PLAIN_TEXT)
        self.assertEqual(test_obj.digest(test_obj._cipher_obj.block_size), self.TEST_MAC_VALUE)

    def test_mac_calculate_hex(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_MAC,
                                             pad_mode=gostcrypto.gostcipher.PAD_MODE_3)
        test_obj.update(self.TEST_PLAIN_TEXT)
        self.assertEqual(test_obj.hexdigest(test_obj._cipher_obj.block_size), self.TEST_MAC_VALUE.hex())

    def test_mac_calculate_double(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_MAC,
                                             pad_mode=gostcrypto.gostcipher.PAD_MODE_3)
        test_obj.update(self.TEST_PLAIN_TEXT)
        test_obj.update(self.TEST_PLAIN_TEXT)
        self.assertEqual(test_obj.digest(test_obj._cipher_obj.block_size), self.TEST_MAC_VALUE_DOUBLE)

    def test_mac_calculate_padding(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_MAC,
                                             pad_mode=gostcrypto.gostcipher.PAD_MODE_3)
        test_obj.update(self.TEST_PLAIN_TEXT_NO_MUL)
        self.assertEqual(test_obj.digest(test_obj._cipher_obj.block_size), self.TEST_MAC_VALUE_PAD)

    def test_mac_calculate_raises(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=self.TEST_INIT_VECT_CTR)
        with self.assertRaises(ValueError) as context:
            test_obj.update(self.TEST_PLAIN_TEXT)
        self.assertTrue('unsupported cipher mode' in str(context.exception))
        with self.assertRaises(ValueError) as context:
            test_obj.digest(test_obj._cipher_obj.block_size)
        self.assertTrue('unsupported cipher mode' in str(context.exception))
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_MAC,
                                              pad_mode=gostcrypto.gostcipher.PAD_MODE_2)
        with self.assertRaises(ValueError) as context:
            test_obj.update(self.TEST_PLAIN_TEXT)
        self.assertTrue('unsupported padding mode' in str(context.exception))
        with self.assertRaises(ValueError) as context:
            test_obj.digest(test_obj._cipher_obj.block_size)
        self.assertTrue('unsupported padding mode' in str(context.exception))
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_MAC,
                                              pad_mode=gostcrypto.gostcipher.PAD_MODE_3)
        with self.assertRaises(ValueError) as context:
            test_obj.update(self.TEST_PLAIN_TEXT)
            test_obj.digest(test_obj._cipher_obj.block_size + 1)
        self.assertTrue('invalid message authentication code size' in str(context.exception))

    def test_decrypt_raises(self):
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_MAC,
                                              pad_mode=gostcrypto.gostcipher.PAD_MODE_3)
        with self.assertRaises(ValueError) as context:
            test_obj.decrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('unsupported cipher mode' in str(context.exception))

    def test_encrypt_raises(self):
        test_obj =  gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_MAC,
                                              pad_mode=gostcrypto.gostcipher.PAD_MODE_3)
        with self.assertRaises(ValueError) as context:
            test_obj.encrypt(self.TEST_PLAIN_TEXT)
        self.assertTrue('unsupported cipher mode' in str(context.exception))

    def test_iv(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_OFB, init_vect=self.TEST_INIT_VECT)
        self.assertEqual(test_obj.iv, self.TEST_INIT_VECT[len(self.TEST_INIT_VECT) - test_obj._cipher_obj.block_size::])

    def test_iv_raises(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB)
        with self.assertRaises(ValueError) as context:
            test_iv =  test_obj.iv
        self.assertTrue('ValueError: invalid IV value' in str(context.exception))

    def test_counter(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=self.TEST_INIT_VECT_CTR)
        test_counter = self.TEST_INIT_VECT_CTR + b'\x00' * (test_obj._cipher_obj.block_size // 2)
        test_counter = bytearray(test_counter)
        self.assertEqual(test_obj.counter, test_counter)

    def test_counter_raises(self):
        test_obj = gostcrypto.gostcipher.new('kuznechik', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB)
        with self.assertRaises(ValueError) as context:
            test_counter =  test_obj.counter
        self.assertTrue('ValueError: invalid counter value' in str(context.exception))

class TestGOST34132015Magma(unittest.TestCase):

    TEST_KEY = bytearray([
        0xff, 0xee, 0xdd, 0xcc, 0xbb, 0xaa, 0x99, 0x88,
        0x77, 0x66, 0x55, 0x44, 0x33, 0x22, 0x11, 0x00,
        0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7,
        0xf8, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff,
    ])

    TEST_PLAIN_TEXT = bytearray([
        0x92, 0xde, 0xf0, 0x6b, 0x3c, 0x13, 0x0a, 0x59,
        0xdb, 0x54, 0xc7, 0x04, 0xf8, 0x18, 0x9d, 0x20,
        0x4a, 0x98, 0xfb, 0x2e, 0x67, 0xa8, 0x02, 0x4c,
        0x89, 0x12, 0x40, 0x9b, 0x17, 0xb5, 0x7e, 0x41,
    ])

    TEST_PLAIN_TEXT_NO_MUL = bytearray([
        0x92, 0xde, 0xf0, 0x6b, 0x3c, 0x13, 0x0a, 0x59,
        0xdb, 0x54, 0xc7, 0x04, 0xf8, 0x18, 0x9d, 0x20,
        0x4a, 0x98, 0xfb, 0x2e, 0x67, 0xa8, 0x02, 0x4c,
        0x89, 0x12, 0x40, 0x9b, 0x17,
    ]) 

    TEST_CIPHER_TEXT_ECB = bytearray([
        0x2b, 0x07, 0x3f, 0x04, 0x94, 0xf3, 0x72, 0xa0,
        0xde, 0x70, 0xe7, 0x15, 0xd3, 0x55, 0x6e, 0x48,
        0x11, 0xd8, 0xd9, 0xe9, 0xea, 0xcf, 0xbc, 0x1e,
        0x7c, 0x68, 0x26, 0x09, 0x96, 0xc6, 0x7e, 0xfb
    ])

    TEST_CIPHER_TEXT_CTR = bytearray([
        0x4e, 0x98, 0x11, 0x0c, 0x97, 0xb7, 0xb9, 0x3c,
        0x3e, 0x25, 0x0d, 0x93, 0xd6, 0xe8, 0x5d, 0x69,
        0x13, 0x6d, 0x86, 0x88, 0x07, 0xb2, 0xdb, 0xef,
        0x56, 0x8e, 0xb6, 0x80, 0xab, 0x52, 0xa1, 0x2d,
    ])

    TEST_CIPHER_TEXT_CTR_NO_MUL = bytearray([
        0x4e, 0x98, 0x11, 0x0c, 0x97, 0xb7, 0xb9, 0x3c,
        0x3e, 0x25, 0x0d, 0x93, 0xd6, 0xe8, 0x5d, 0x69,
        0x13, 0x6d, 0x86, 0x88, 0x07, 0xb2, 0xdb, 0xef,
        0x56, 0x8e, 0xb6, 0x80, 0xab,
    ])

    TEST_CIPHER_TEXT_CBC = bytearray([
        0x96, 0xd1, 0xb0, 0x5e, 0xea, 0x68, 0x39, 0x19,
        0xaf, 0xf7, 0x61, 0x29, 0xab, 0xb9, 0x37, 0xb9,
        0x50, 0x58, 0xb4, 0xa1, 0xc4, 0xbc, 0x00, 0x19,
        0x20, 0xb7, 0x8b, 0x1a, 0x7c, 0xd7, 0xe6, 0x67,
    ])

    TEST_CIPHER_TEXT_OFB = bytearray([
        0xdb, 0x37, 0xe0, 0xe2, 0x66, 0x90, 0x3c, 0x83,
        0x0d, 0x46, 0x64, 0x4c, 0x1f, 0x9a, 0x08, 0x9c,
        0xa0, 0xf8, 0x30, 0x62, 0x43, 0x0e, 0x32, 0x7e,
        0xc8, 0x24, 0xef, 0xb8, 0xbd, 0x4f, 0xdb, 0x05,
    ])

    TEST_CIPHER_TEXT_OFB_NO_MUL = bytearray([
        0xdb, 0x37, 0xe0, 0xe2, 0x66, 0x90, 0x3c, 0x83,
        0x0d, 0x46, 0x64, 0x4c, 0x1f, 0x9a, 0x08, 0x9c,
        0xa0, 0xf8, 0x30, 0x62, 0x43, 0x0e, 0x32, 0x7e,
        0xc8, 0x24, 0xef, 0xb8, 0xbd,
    ])

    TEST_CIPHER_TEXT_CFB = bytearray([
        0xdb, 0x37, 0xe0, 0xe2, 0x66, 0x90, 0x3c, 0x83,
        0x0d, 0x46, 0x64, 0x4c, 0x1f, 0x9a, 0x08, 0x9c,
        0x24, 0xbd, 0xd2, 0x03, 0x53, 0x15, 0xd3, 0x8b,
        0xbc, 0xc0, 0x32, 0x14, 0x21, 0x07, 0x55, 0x05,
    ])

    TEST_CIPHER_TEXT_CFB_NO_MUL = bytearray([
        0xdb, 0x37, 0xe0, 0xe2, 0x66, 0x90, 0x3c, 0x83,
        0x0d, 0x46, 0x64, 0x4c, 0x1f, 0x9a, 0x08, 0x9c,
        0x24, 0xbd, 0xd2, 0x03, 0x53, 0x15, 0xd3, 0x8b,
        0xbc, 0xc0, 0x32, 0x14, 0x21,
    ])

    TEST_INIT_VECT = bytearray([
        0x12, 0x34, 0x56, 0x78, 0x90, 0xab, 0xcd, 0xef,  
        0x23, 0x45, 0x67, 0x89, 0x0a, 0xbc, 0xde, 0xf1,
    ])

    TEST_INIT_VECT_CBC = bytearray([
        0x12, 0x34, 0x56, 0x78, 0x90, 0xab, 0xcd, 0xef,  
        0x23, 0x45, 0x67, 0x89, 0x0a, 0xbc, 0xde, 0xf1,  
        0x34, 0x56, 0x78, 0x90, 0xab, 0xcd, 0xef, 0x12,
    ])

    TEST_INIT_VECT_CTR = bytearray([
        0x12, 0x34, 0x56, 0x78,
    ])

    TEST_MAC_VALUE = bytearray([
        0x15, 0x4e, 0x72, 0x10, 0x20, 0x30, 0xc5, 0xbb,
    ])

    def test_ecb_encrypt(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT), self.TEST_CIPHER_TEXT_ECB)

    def test_ecb_decrypt(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB)
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_ECB), self.TEST_PLAIN_TEXT)

    def test_ctr_encrypt(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=self.TEST_INIT_VECT_CTR)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT), self.TEST_CIPHER_TEXT_CTR)
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=self.TEST_INIT_VECT_CTR)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT_NO_MUL), self.TEST_CIPHER_TEXT_CTR_NO_MUL)

    def test_ctr_decrypt(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=self.TEST_INIT_VECT_CTR)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_CTR), self.TEST_PLAIN_TEXT)
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=self.TEST_INIT_VECT_CTR)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_CTR_NO_MUL), self.TEST_PLAIN_TEXT_NO_MUL)

    def test_cbc_encrypt(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_CBC, init_vect=self.TEST_INIT_VECT_CBC)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT), self.TEST_CIPHER_TEXT_CBC)

    def test_cbc_decrypt(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_CBC, init_vect=self.TEST_INIT_VECT_CBC)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_CBC), self.TEST_PLAIN_TEXT)

    def test_cfb_encrypt(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_CFB, init_vect=self.TEST_INIT_VECT)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT), self.TEST_CIPHER_TEXT_CFB)
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_CFB, init_vect=self.TEST_INIT_VECT)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT_NO_MUL), self.TEST_CIPHER_TEXT_CFB_NO_MUL)

    def test_cfb_decrypt(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_CFB, init_vect=self.TEST_INIT_VECT)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_CFB), self.TEST_PLAIN_TEXT)
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_CFB, init_vect=self.TEST_INIT_VECT)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_CFB_NO_MUL), self.TEST_PLAIN_TEXT_NO_MUL)

    def test_ofb_encrypt(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_OFB, init_vect=self.TEST_INIT_VECT)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT), self.TEST_CIPHER_TEXT_OFB)
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_OFB, init_vect=self.TEST_INIT_VECT)
        self.assertEqual(test_obj.encrypt(self.TEST_PLAIN_TEXT_NO_MUL), self.TEST_CIPHER_TEXT_OFB_NO_MUL)

    def test_ofb_decrypt(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_OFB, init_vect=self.TEST_INIT_VECT)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_OFB), self.TEST_PLAIN_TEXT)
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_OFB, init_vect=self.TEST_INIT_VECT)    
        self.assertEqual(test_obj.decrypt(self.TEST_CIPHER_TEXT_OFB_NO_MUL), self.TEST_PLAIN_TEXT_NO_MUL)

    def test_mac_calculate(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_MAC,
                                             pad_mode=gostcrypto.gostcipher.PAD_MODE_3)
        test_obj.update(self.TEST_PLAIN_TEXT)                                             
        self.assertEqual(test_obj.digest(test_obj._cipher_obj.block_size), self.TEST_MAC_VALUE)

    def test_iv(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_OFB, init_vect=self.TEST_INIT_VECT)
        self.assertEqual(test_obj.iv, self.TEST_INIT_VECT[len(self.TEST_INIT_VECT) - test_obj._cipher_obj.block_size::])

    def test_iv_raises(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB)
        with self.assertRaises(ValueError) as context:
            test_iv =  test_obj.iv
        self.assertTrue('invalid IV value' in str(context.exception))

    def test_counter(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_CTR, init_vect=self.TEST_INIT_VECT_CTR)
        test_counter = self.TEST_INIT_VECT_CTR + b'\x00' * (test_obj._cipher_obj.block_size // 2)
        test_counter = bytearray(test_counter)
        self.assertEqual(test_obj.counter, test_counter)

    def test_counter_raises(self):
        test_obj = gostcrypto.gostcipher.new('magma', self.TEST_KEY, gostcrypto.gostcipher.MODE_ECB)
        with self.assertRaises(ValueError) as context:
            test_iv =  test_obj.counter
        self.assertTrue('invalid counter value' in str(context.exception))

if __name__ == '__main__':
    unittest.main()



import unittest
import os
from unittest import mock

import gostcrypto

TEST_PASSWORD = b'password'
TEST_SALT = b'salt'
TEST_DK_1 = bytearray.fromhex('64770af7f748c3b1c9ac831dbcfd85c26111b30a8a657ddc3056b80ca73e040d2854fd36811f6d825cc4ab66ec0a68a490a9e5cf5156b3a2b7eecddbf9a16b47')
TEST_DK_2 = bytearray.fromhex('5a585bafdfbb6e8830d6d68aa3b43ac00d2e4aebce01c9b31c2caed56f0236d4d34b2b8fbd2c4e89d54d46f50e47d45bbac301571743119e8d3c42ba66d348de')
TEST_DK_SHOT = bytearray.fromhex('64770af7f748c3b1c9ac831dbcfd85c26111b30a8a657ddc3056b80ca73e040d')

def os_urandom(value):
    return TEST_SALT

class TestHMAC(unittest.TestCase):

    def test_pbcdf_derive_1(self):
        test_pbkdf = gostcrypto.gostpbkdf.new(TEST_PASSWORD, TEST_SALT, 1)
        self.assertEqual(test_pbkdf.derive(64), TEST_DK_1)
        test_pbkdf = gostcrypto.gostpbkdf.new(TEST_PASSWORD, TEST_SALT, 1)
        self.assertEqual(test_pbkdf.hexderive(64), TEST_DK_1.hex())

    def test_pbcdf_derive_2(self):
        test_pbkdf = gostcrypto.gostpbkdf.new(TEST_PASSWORD, TEST_SALT, 2)
        self.assertEqual(test_pbkdf.derive(64), TEST_DK_2)
        test_pbkdf = gostcrypto.gostpbkdf.new(TEST_PASSWORD, TEST_SALT, 2)
        self.assertEqual(test_pbkdf.hexderive(64), TEST_DK_2.hex())

    def test_pbcdf_derive_shot(self):
        test_pbkdf = gostcrypto.gostpbkdf.new(TEST_PASSWORD, TEST_SALT, 1)
        self.assertEqual(test_pbkdf.derive(32), TEST_DK_SHOT)
        test_pbkdf = gostcrypto.gostpbkdf.new(TEST_PASSWORD, TEST_SALT, 1)
        self.assertEqual(test_pbkdf.hexderive(32), TEST_DK_SHOT.hex())

    def test_pbcdf_derive_raises(self):
        test_pbkdf = gostcrypto.gostpbkdf.new(TEST_PASSWORD, TEST_SALT, 1)
        with self.assertRaises(ValueError) as context:
            test_pbkdf.hexderive(((2 ** 32 - 1) * 64) + 1)
        self.assertTrue('invalid size of the derived key' in str(context.exception))
        with self.assertRaises(ValueError) as context:
            test_pbkdf.derive(((2 ** 32 - 1) * 64) + 1)
        self.assertTrue('invalid size of the derived key' in str(context.exception))

    def test_pbcdf_raises(self):
        with self.assertRaises(ValueError) as context:
            test_pbkdf = gostcrypto.gostpbkdf.R5011112016(None, TEST_SALT, 1)
        self.assertTrue('invalid password value' in str(context.exception))

    def test_pbcdf_salt(self):
        test_pbkdf = gostcrypto.gostpbkdf.new(TEST_PASSWORD, TEST_SALT, 1)
        self.assertEqual(test_pbkdf.salt, TEST_SALT)

    def test_pbkdf_urandom(self):
        with mock.patch('os.urandom', os_urandom):
            test_pbkdf = gostcrypto.gostpbkdf.new(TEST_PASSWORD, None, 1)
        self.assertEqual(test_pbkdf.derive(32), TEST_DK_SHOT)



"""The module that implements various block encryption modes (ECB, CBC, CFB, OFB, CTR and
   MAC according to GOST 34.13-2015.
"""

__title__ = 'gostcipher'
__author__ = 'Evgeny Drobotun'
__author_email__ = 'drobotun@xakep.ru'
__license__ = 'MIT'
__copyright__ = 'Copyright (C) 2020 Evgeny Drobotun'

from .gost_34_12_2015 import *
from .gost_34_13_2015 import *
